# -*- coding: utf-8 -*-
from abjad.tools import durationtools
from abjad.tools import mathtools
from abjad.tools import selectiontools
from abjad.tools import sequencetools
from abjad.tools import systemtools
from abjad.tools.topleveltools import inspect_
from abjad.tools.topleveltools import iterate
from abjad.tools.scoretools.Component import Component


class Container(Component):
    r'''A container.

    ..  container:: example

        Intializes from string:

        ::

            >>> container = Container("c'4 e'4 d'4 e'8 f'8")
            >>> show(container) # doctest: +SKIP

        ..  doctest::

            >>> f(container)
            {
                c'4
                e'4
                d'4
                e'8
                f'8
            }

    ..  container:: example

        Intializes from components:

        ::

            >>> notes = [
            ...     Note("c'4"),
            ...     Note("e'4"),
            ...     Note("d'4"),
            ...     Note("e'8"),
            ...     Note("f'8"),
            ...     ]
            >>> container = Container(notes)
            >>> show(container) # doctest: +SKIP

        ..  doctest::

            >>> f(container)
            {
                c'4
                e'4
                d'4
                e'8
                f'8
            }

    ..  container:: example

        Intializes from selections:

        ::

            >>> notes = [
            ...     Note("c'4"),
            ...     Note("e'4"),
            ...     Note("d'4"),
            ...     Note("e'8"),
            ...     Note("f'8"),
            ...     ]
            >>> selection = select(notes)
            >>> container = Container(selection)
            >>> show(container) # doctest: +SKIP

        ..  doctest::

            >>> f(container)
            {
                c'4
                e'4
                d'4
                e'8
                f'8
            }

    ..  container:: example

        Intializes from mixed components and selections:

        ::

            >>> items = [
            ...     Note("c'4"),
            ...     select(Note("e'4")),
            ...     select(Note("d'4")),
            ...     Note("e'8"),
            ...     Note("f'8"),
            ...     ]
            >>> container = Container(items)
            >>> show(container) # doctest: +SKIP

        ..  doctest::

            >>> f(container)
            {
                c'4
                e'4
                d'4
                e'8
                f'8
            }

    ..  container:: example

        Containers are iterables:

        ::

            >>> import collections
            >>> container = Container("c'4 e'4 d'4 e'8 f'8")
            >>> isinstance(container, collections.Iterable)
            True

    ..  container:: example

        Containers are not sequences because containers do not implement
        reverse:

        ::

            >>> container = Container("c'4 e'4 d'4 e'8 f'8")
            >>> isinstance(container, collections.Sequence)
            False

    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Containers'

    __slots__ = (
        '_formatter',
        '_music',
        '_named_children',
        '_name',
        '_is_simultaneous',
        )

    ### INITIALIZER ###

    def __init__(self, music=None, is_simultaneous=None, name=None):
        music = music or []
        Component.__init__(self, name=name)
        self._named_children = {}
        self._is_simultaneous = False
        self._initialize_music(music)
        self.is_simultaneous = is_simultaneous

    ### SPECIAL METHODS ###

    def __contains__(self, argument):
        r'''Is true when `argument` appears in container.
        Otherwise false.

        Returns true or false.
        '''
        if isinstance(argument, str):
            return argument in self._named_children
        else:
            for x in self._music:
                if x is argument:
                    return True
            else:
                return False

    def __delitem__(self, i):
        r'''Deletes components(s) at index `i` in container.

        ..  container:: example

            Deletes first tuplet in voice:

            ::

                >>> voice = Voice()
                >>> voice.append(Tuplet((2, 3), "c'4 d'4 e'4"))
                >>> voice.append(Tuplet((2, 3), "e'4 d'4 c'4"))
                >>> leaves = iterate(voice).by_leaf()
                >>> attach(Slur(), list(leaves))
                >>> show(voice) # doctest: +SKIP

            ..  doctest::

                >>> f(voice)
                \new Voice {
                    \times 2/3 {
                        c'4 (
                        d'4
                        e'4
                    }
                    \times 2/3 {
                        e'4
                        d'4
                        c'4 )
                    }
                }

            ::

                >>> tuplet_1 = voice[0]
                >>> del(voice[0])

            First tuplet no longer appears in voice:

                >>> show(voice) # doctest: +SKIP

            ..  doctest::

                >>> f(voice)
                \new Voice {
                    \times 2/3 {
                        e'4 (
                        d'4
                        c'4 )
                    }
                }

            ::

                >>> inspect_(voice).is_well_formed()
                True

            First tuplet is no longer slurred but is still well-formed:

            ::

                >>> show(tuplet_1) # doctest: +SKIP

            ..  doctest::

                >>> f(tuplet_1)
                \times 2/3 {
                    c'4
                    d'4
                    e'4
                }

            ::

                >>> inspect_(tuplet_1).is_well_formed()
                True

        Withdraws component(s) from crossing spanners.

        Preserves spanners that component(s) cover(s).

        Returns none.
        '''
        components = self[i]
        if not isinstance(components, selectiontools.Selection):
            components = selectiontools.Selection([components])
        if not self.is_simultaneous:
            components._withdraw_from_crossing_spanners()
        components._set_parents(None)

    def __getitem__(self, argument):
        r'''Gets item or slice identified by `argument`.

        Traverses top-level items only.

        Returns component.
        '''
        if isinstance(argument, int):
            return self._music.__getitem__(argument)
        elif isinstance(argument, slice) and not self.is_simultaneous:
            return selectiontools.Selection(self._music.__getitem__(argument))
        elif isinstance(argument, slice) and self.is_simultaneous:
            return selectiontools.Selection(self._music.__getitem__(argument))
        elif isinstance(argument, str):
            if argument not in self._named_children:
                message = 'can not find component named {!r}.'
                message = message.format(argument)
                raise ValueError(message)
            elif 1 < len(self._named_children.__getitem__(argument)):
                message = 'multiple components named {!r}.'
                message = message.format(argument)
                raise ValueError(message)
            return self._named_children.__getitem__(argument)[0]
        message = 'can not get container at {!r}.'
        message = message.format(argument)
        raise ValueError(message)

    def __graph__(self, spanner=None, **keywords):
        r'''Graphviz graph representation of container.

        Returns Graphviz graph.
        '''
        def recurse(component, leaf_cluster):
            component_node = component._as_graphviz_node()
            node_mapping[component] = component_node
            node_order = [component_node.name]
            if isinstance(component, scoretools.Container):
                graph.append(component_node)
                this_leaf_cluster = graphtools.GraphvizSubgraph(
                    name=component_node.name,
                    attributes={
                        'color': 'grey75',
                        'penwidth': 2,
                        },
                    )
                all_are_leaves = True
                pending_node_order = []
                for child in component:
                    if not isinstance(child, scoretools.Leaf):
                        all_are_leaves = False
                    child_node, child_node_order = recurse(
                        child, this_leaf_cluster)
                    pending_node_order.extend(child_node_order)
                    edge = graphtools.GraphvizEdge()
                    edge.attach(component_node, child_node)
                if all_are_leaves:
                    pending_node_order.reverse()
                node_order.extend(pending_node_order)
                if len(this_leaf_cluster):
                    leaf_cluster.append(this_leaf_cluster)
            else:
                leaf_cluster.append(component_node)
            return component_node, node_order

        from abjad.tools import graphtools
        from abjad.tools import scoretools
        node_order = []
        node_mapping = {}
        graph = graphtools.GraphvizGraph(
            name='G',
            attributes={
                'style': 'rounded',
                },
            edge_attributes={
                },
            node_attributes={
                'fontname': 'Arial',
                'shape': 'none',
                },
            )
        leaf_cluster = graphtools.GraphvizSubgraph(name='leaves')
        component_node, node_order = recurse(self, leaf_cluster)
        if len(leaf_cluster) == 1:
            graph.append(leaf_cluster[0])
        elif len(leaf_cluster):
            graph.append(leaf_cluster)
        graph._node_order = node_order

        if spanner:
            pairs = sequencetools.Sequence(spanner.components).nwise()
            for component_one, component_two in pairs:
                node_one = node_mapping[component_one]
                node_two = node_mapping[component_two]
                edge = graphtools.GraphvizEdge(
                    attributes={
                        'constraint': False,
                        'penwidth': 5,
                        },
                    )
                edge.attach(node_one, node_two)
            for component in spanner.components:
                node = node_mapping[component]
                table = node[0]
                table.attributes['border'] = 4
                table.attributes['bgcolor'] = 'grey80'
                if isinstance(component, Container):
                    for child in iterate(component).depth_first():
                        if child is component:
                            continue
                        node = node_mapping[child]
                        table = node[0]
                        table.attributes['bgcolor'] = 'grey80'

        return graph

    def __iter__(self):
        r'''Iterates container.

        ..  container:: example

            Abjad containers are iterables:

            ::

                >>> import collections
                >>> container = Container()
                >>> isinstance(container, collections.Iterable)
                True

        ..  container:: example

            Abjad containers are not sequences:

            ::

                >>> import collections
                >>> container = Container()
                >>> isinstance(container, collections.Sequence)
                False

        Yields container elements.

        Returns generator.
        '''
        return iter(self._music)

    def __len__(self):
        r'''Gets number of items in container.

        Returns nonnegative integer.
        '''
        return len(self._music)

    def __setitem__(self, i, argument):
        r'''Sets container `i` equal to `argument`.
        Finds spanners that dominate self[i] and children of self[i].
        Replaces contents at self[i] with 'argument'.
        Reattaches spanners to new contents.
        Always leaves score tree in tact.

        Returns none.
        '''
        return self._set_item(i, argument)

    ### PRIVATE METHODS ###

    def _append_without_withdrawing_from_crossing_spanners(self, component):
        '''Not composer-safe.
        '''
        self._set_item(slice(len(self), len(self)), [component],
            withdraw_components_from_crossing_spanners=False)

    def _as_graphviz_node(self):
        from abjad.tools import graphtools
        node = Component._as_graphviz_node(self)
        node[0].append(
            graphtools.GraphvizTableRow([
                graphtools.GraphvizTableCell(
                    label=type(self).__name__,
                    attributes={'border': 0},
                    ),
                ])
            )
        return node

    def _copy_with_children_and_indicators_but_without_spanners(self):
        new = self._copy_with_indicators_but_without_children_or_spanners()
        for component in self:
            new_component = \
                component._copy_with_children_and_indicators_but_without_spanners()
            new.append(new_component)
        return new

    def _copy_with_indicators_but_without_children_or_spanners(self):
        new = Component._copy_with_indicators_but_without_children_or_spanners(
            self)
        new.is_simultaneous = self.is_simultaneous
        return new

    def _eject_contents(self):
        if inspect_(self).get_parentage().parent is not None:
            message = 'can not eject contents of in-score container.'
            raise Exception(message)
        contents = self[:]
        for component in contents:
            component._set_parent(None)
        self._music[:] = []
        return contents

    def _format_after_slot(self, bundle):
        result = []
        result.append(('commands', bundle.after.commands))
        result.append(('comments', bundle.after.comments))
        return tuple(result)

    def _format_before_slot(self, bundle):
        result = []
        result.append(('comments', bundle.before.comments))
        result.append(('commands', bundle.before.commands))
        return tuple(result)

    def _format_close_brackets_slot(self, bundle):
        result = []
        if self.is_simultaneous:
            brackets_close = ['>>']
        else:
            brackets_close = ['}']
        result.append([('close brackets', ''), brackets_close])
        return tuple(result)

    def _format_closing_slot(self, bundle):
        result = []
        result.append(('grob reverts', bundle.grob_reverts))
        result.append(('commands', bundle.closing.commands))
        result.append(('comments', bundle.closing.comments))
        return self._format_slot_contributions_with_indent(result)

    def _format_content_pieces(self):
        from abjad.tools import systemtools
        indent = systemtools.LilyPondFormatManager.indent
        result = []
        for m in self._music:
            result.extend(format(m).split('\n'))
        result = [indent + x for x in result]
        return result

    def _format_contents_slot(self, bundle):
        result = []
        result.append([('contents', '_contents'), self._format_content_pieces()])
        return tuple(result)

    def _format_open_brackets_slot(self, bundle):
        result = []
        if self.is_simultaneous:
            brackets_open = ['<<']
        else:
            brackets_open = ['{']
        result.append([('open brackets', ''), brackets_open])
        return tuple(result)

    def _format_opening_slot(self, bundle):
        result = []
        result.append(('comments', bundle.opening.comments))
        result.append(('commands', bundle.opening.commands))
        result.append(('grob overrides', bundle.grob_overrides))
        result.append(('context settings', bundle.context_settings))
        return self._format_slot_contributions_with_indent(result)

    def _format_slot_contributions_with_indent(self, slot):
        from abjad.tools import systemtools
        indent = systemtools.LilyPondFormatManager.indent
        result = []
        for contributor, contributions in slot:
            result.append(
                (contributor, tuple([indent + x for x in contributions])))
        return tuple(result)

    def _get_abbreviated_string_format(self):
        if 0 < len(self):
            summary = str(len(self))
        else:
            summary = ''
        if self.is_simultaneous:
            open_bracket_string, close_bracket_string = '<<', '>>'
        else:
            open_bracket_string, close_bracket_string = '{', '}'
        name = self.name
        if name is not None:
            name = '-"{}"'.format(name)
        else:
            name = ''
        if hasattr(self, '_context_name'):
            result = '<{}{}{}{}{}>'
            result = result.format(
                self.context_name,
                name,
                open_bracket_string,
                summary,
                close_bracket_string,
                )
        else:
            result = '<{}{}{}{}>'
            result = result.format(
                name,
                open_bracket_string,
                summary,
                close_bracket_string,
                )
        return result

    def _get_format_specification(self):
        from abjad.tools import scoretools
        repr_text = None
        repr_args_values = []
        repr_kwargs_names = []
        if self.is_simultaneous:
            repr_kwargs_names.append('is_simultaneous')
        storage_format_args_values = []
        if self:
            repr_args_values.append(self._contents_summary)
            lilypond_format = ' '.join(format(x, 'lilypond') for x in self)
            lilypond_format = lilypond_format.replace('\n', ' ')
            lilypond_format = lilypond_format.replace('\t', ' ')
            lilypond_format = lilypond_format.replace('  ', ' ')
            storage_format_args_values.append(lilypond_format)
            if not all(isinstance(x, scoretools.Leaf) for x in self):
                repr_text = self._get_abbreviated_string_format()
        return systemtools.FormatSpecification(
            client=self,
            repr_args_values=repr_args_values,
            repr_kwargs_names=repr_kwargs_names,
            repr_text=repr_text,
            storage_format_args_values=storage_format_args_values,
            storage_format_kwargs_names=[],
            )

    def _get_spanners_that_dominate_component_pair(self, left, right):
        r'''Returns spanners that dominant component pair.
        Returns set (spanner, index) pairs.
        `left` must be an Abjad component or None.
        `right` must be an Abjad component or None.

        If both `left` and `right` are components,
        then `left` and `right` must be logical-voice-contiguous.

        This is a version of Selection._get_dominant_spanners().
        This version is useful for finding spanners that dominant
        a zero-length slice between components, as in staff[2:2].
        '''
        if left is None or right is None:
            return set([])
        left_contained = left._get_descendants()._get_spanners()
        right_contained = right._get_descendants()._get_spanners()
        dominant_spanners = left_contained & right_contained
        right_start_offset = right._get_timespan().start_offset
        components_after_gap = []
        for component in right._get_lineage():
            if component._get_timespan().start_offset == right_start_offset:
                components_after_gap.append(component)
        receipt = set([])
        for spanner in dominant_spanners:
            for component in components_after_gap:
                if component in spanner:
                    index = spanner._index(component)
                    receipt.add((spanner, index))
                    continue
        return receipt

    def _get_spanners_that_dominate_slice(self, start, stop):
        if start == stop:
            if start == 0:
                left = None
            else:
                left = self[start - 1]
            if len(self) <= stop:
                right = None
            else:
                right = self[stop]
            if left is None:
                left = self._get_sibling(-1)
            if right is None:
                right = self._get_sibling(1)
            spanners_receipt = \
                self._get_spanners_that_dominate_component_pair(left, right)
        else:
            selection = self[start:stop]
            spanners_receipt = selection._get_dominant_spanners()
        return spanners_receipt

    def _get_spanners_that_span_slice(self, start, stop):
        if start == stop:
            if start == 0:
                left = None
            else:
                left = self[start - 1]
            if len(self) <= stop:
                right = None
            else:
                right = self[stop]
            if left is None:
                left = self._get_sibling(-1)
            if right is None:
                right = self._get_sibling(1)
            print(left, right)
        else:
            selection = self[start:stop]
            print(selection)

    def _iterate_bottom_up(self):
        def recurse(node):
            if isinstance(node, Container):
                for x in node:
                    for y in recurse(x):
                        yield y
            yield node
        return recurse(self)

    def _iterate_top_down(self):
        def recurse(node):
            yield node
            if isinstance(node, Container):
                for x in node:
                    for y in recurse(x):
                        yield y
        return recurse(self)

    def _scale_contents(self, multiplier):
        for argument in iterate(self[:]).by_topmost_logical_ties_and_components():
            argument._scale(multiplier)

    def _set_item(
        self,
        i,
        argument,
        withdraw_components_from_crossing_spanners=True,
        ):
        r'''This method exists because __setitem__ can not accept keywords.

        Note that setting
        withdraw_components_from_crossing_spanners=False constitutes a
        composer-unsafe use of this method.

        Only private methods should set this keyword.
        '''
        from abjad.tools import scoretools
        from abjad.tools import selectiontools
        # cache indicators attached to components in argument
        expr_indicators = []
        for component in iterate(argument).by_class():
            indicators = component._get_indicators(unwrap=False)
            expr_indicators.extend(indicators)
        # item assignment
        if isinstance(i, int):
            if isinstance(argument, str):
                argument = self._parse_string(argument)[:]
                assert len(argument) == 1, repr(argument)
                argument = argument[0]
            else:
                argument = [argument]
            if i < 0:
                i = len(self) + i
            i = slice(i, i + 1)
        else:
            if isinstance(argument, str):
                argument = self._parse_string(argument)[:]
            elif (isinstance(argument, list) and
                len(argument) == 1 and
                isinstance(argument[0], str)):
                argument = self._parse_string(argument[0])[:]
        prototype = (scoretools.Component, selectiontools.Selection)
        assert all(isinstance(x, prototype) for x in argument)
        new_expr = []
        for item in argument:
            if isinstance(item, selectiontools.Selection):
                new_expr.extend(item)
            else:
                new_expr.append(item)
        argument = new_expr
        assert all(isinstance(x, scoretools.Component) for x in argument)
        if any(isinstance(x, scoretools.GraceContainer) for x in argument):
            message = 'must attach grace container to note or chord.'
            raise Exception(message)
        if self._check_for_cycles(argument):
            raise ParentageError('Attempted to induce cycles.')
        if (
            i.start == i.stop and
            i.start is not None and
            i.stop is not None and
            i.start <= -len(self)
            ):
            start, stop = 0, 0
        else:
            start, stop, stride = i.indices(len(self))
        old = self[start:stop]
        spanners_receipt = self._get_spanners_that_dominate_slice(start, stop)
        #print('RECEIPT', spanners_receipt, self, argument)
        for component in old:
            for child in iterate([component]).by_class():
                for spanner in child._get_spanners():
                    spanner._remove(child)
        del(self[start:stop])
        # must withdraw before setting in self!
        # otherwise circular withdraw ensues!
        if withdraw_components_from_crossing_spanners:
            selection = selectiontools.Selection(argument)
            if selection._all_are_contiguous_components_in_same_logical_voice(
                selection):
                selection._withdraw_from_crossing_spanners()
        self._music.__setitem__(slice(start, start), argument)
        for component in argument:
            component._set_parent(self)
        for spanner, index in spanners_receipt:
            for component in reversed(argument):
                spanner._insert(index, component)
                component._spanners.add(spanner)
        for indicator in expr_indicators:
            if hasattr(indicator, '_update_effective_context'):
                indicator._update_effective_context()

    ### PRIVATE METHODS ###

    @staticmethod
    def _all_are_orphan_components(argument):
        from abjad.tools import scoretools
        for component in argument:
            if not isinstance(component, scoretools.Component):
                return False
            if not component._get_parentage().is_orphan:
                return False
        return True

    @staticmethod
    def _flatten_selections(music):
        components = []
        for item in music:
            if isinstance(item, selectiontools.Selection):
                components.extend(item)
            else:
                components.append(item)
        return components

    def _initialize_music(self, music):
        Selection = selectiontools.Selection
        music = music or []
        if isinstance(music, list):
            music = self._flatten_selections(music)
        if self._all_are_orphan_components(music):
            self._music = list(music)
            self[:]._set_parents(self)
        elif Selection._all_are_contiguous_components_in_same_logical_voice(
            music):
            music = selectiontools.Selection(music)
            parent, start, stop = music._get_parent_and_start_stop_indices()
            self._music = list(music)
            self[:]._set_parents(self)
            assert parent is not None
            parent._music.insert(start, self)
            self._set_parent(parent)
        elif isinstance(music, str):
            parsed = self._parse_string(music)
            self._music = []
            self.is_simultaneous = parsed.is_simultaneous
            if (parsed.is_simultaneous or
                not Selection._all_are_contiguous_components_in_same_logical_voice(
                    parsed[:])):
                while len(parsed):
                    self.append(parsed.pop(0))
            else:
                self[:] = parsed[:]
        else:
            message = 'can not initialize container from {!r}.'
            message = message.format((music))
            raise TypeError(message)

    def _is_one_of_my_first_leaves(self, leaf):
        return leaf in self._get_descendants_starting_with()

    def _is_one_of_my_last_leaves(self, leaf):
        return leaf in self._get_descendants_stopping_with()

    def _move_spanners_to_children(self):
        for spanner in self._get_spanners():
            i = spanner._index(self)
            spanner._components.__setitem__(slice(i, i + 1), self[:])
            for component in self:
                component._spanners.add(spanner)
            self._spanners.discard(spanner)
        return self

    def _parse_string(self, string):
        from abjad.tools import lilypondfiletools
        from abjad.tools import lilypondparsertools
        from abjad.tools import rhythmtreetools
        from abjad.tools.topleveltools import parse
        user_input = string.strip()
        if user_input.startswith('abj:'):
            parser = lilypondparsertools.ReducedLyParser()
            parsed = parser(user_input[4:])
            if parser._toplevel_component_count == 1:
                parsed = Container([parsed])
        elif user_input.startswith('rtm:'):
            parsed = rhythmtreetools.parse_rtm_syntax(user_input[4:])
        else:
            if (
                not user_input.startswith('<<') or
                not user_input.endswith('>>')
                ):
                user_input = '{{ {} }}'.format(user_input)
            parsed = parse(user_input)
            if isinstance(parsed, lilypondfiletools.LilyPondFile):
                parsed = Container(parsed.items[:])
            assert isinstance(parsed, Container)
        return parsed

    def _scale(self, multiplier):
        self._scale_contents(multiplier)

    def _split_at_index(self, i, fracture_spanners=False):
        r'''Splits container to the left of index `i`.

        Preserves tuplet multiplier when container is a tuplet.

        Preserves time signature denominator when container is a measure.

        Resizes resizable containers.

        Returns split parts.
        '''
        from abjad.tools import indicatortools
        from abjad.tools import scoretools
        from abjad.tools import selectiontools
        # partition my music
        left_music = self[:i]
        right_music = self[i:]
        # instantiate new left and right containers
        if isinstance(self, scoretools.Measure):
            time_signature = self._get_effective(
                indicatortools.TimeSignature)
            denominator = time_signature.denominator
            left_duration = sum([x._get_duration() for x in left_music])
            left_pair = mathtools.NonreducedFraction(left_duration)
            left_pair = left_pair.with_multiple_of_denominator(denominator)
            left_time_signature = indicatortools.TimeSignature(left_pair)
            left = type(self)(left_time_signature, left_music)
            left.implicit_scaling = self.implicit_scaling
            right_duration = sum([x._get_duration() for x in right_music])
            right_pair = mathtools.NonreducedFraction(right_duration)
            right_pair = right_pair.with_multiple_of_denominator(denominator)
            right_time_signature = indicatortools.TimeSignature(right_pair)
            right = type(self)(right_time_signature, right_music)
            right.implicit_scaling = self.implicit_scaling
        elif isinstance(self, scoretools.FixedDurationTuplet):
            multiplier = self.multiplier
            left = type(self)(1, left_music)
            right = type(self)(1, right_music)
            target_duration = multiplier * left._contents_duration
            left.target_duration = target_duration
            target_duration = multiplier * right._contents_duration
            right.target_duration = target_duration
        elif isinstance(self, scoretools.Tuplet):
            multiplier = self.multiplier
            left = type(self)(multiplier, left_music)
            right = type(self)(multiplier, right_music)
        else:
            left = type(self)(left_music)
            right = type(self)(right_music)
        # save left and right containers together for iteration
        halves = (left, right)
        nonempty_halves = [half for half in halves if len(half)]
        # give my attached spanners to my children
        self._move_spanners_to_children()
        # incorporate left and right parents in score if possible
        selection = selectiontools.Selection(self)
        parent, start, stop = selection._get_parent_and_start_stop_indices()
        if parent is not None:
            parent._music.__setitem__(slice(start, stop + 1), nonempty_halves)
            for part in nonempty_halves:
                part._set_parent(parent)
        else:
            left._set_parent(None)
            right._set_parent(None)
        # fracture spanners if requested
        if fracture_spanners:
            for spanner in left._get_spanners():
                index = spanner._index(left)
                spanner._fracture(index, direction=Right)
        # return new left and right containers
        return halves

    def _split_by_duration(
        self,
        duration,
        fracture_spanners=False,
        tie_split_notes=True,
        use_messiaen_style_ties=False,
        ):
        from abjad.tools import scoretools
        from abjad.tools import selectiontools
        # check input
        duration = durationtools.Duration(duration)
        assert 0 <= duration, repr(duration)
        # if zero duration then return empty list and self
        if duration == 0:
            return [], self
        # get split point score offset
        global_split_point = self._get_timespan().start_offset + duration
        # get any duration-crossing descendents
        cross_offset = self._get_timespan().start_offset + duration
        duration_crossing_descendants = []
        for descendant in self._get_descendants():
            start_offset = descendant._get_timespan().start_offset
            stop_offset = descendant._get_timespan().stop_offset
            if start_offset < cross_offset < stop_offset:
                duration_crossing_descendants.append(descendant)
        # get any duration-crossing measure descendents
        measures = [
            x for x in duration_crossing_descendants
            if isinstance(x, scoretools.Measure)
            ]
        # if we must split a power-of-two measure at non-power-of-two
        # split point then go ahead and transform the power-of-two measure
        # to non-power-of-two equivalent now;
        # code that crawls and splits later on will be happier
        if len(measures) == 1:
            measure = measures[0]
            split_point_in_measure = \
                global_split_point - measure._get_timespan().start_offset
            if measure.has_non_power_of_two_denominator:
                if not measure.implied_prolation == \
                    split_point_in_measure.implied_prolation:
                    raise NotImplementedError
            elif not mathtools.is_nonnegative_integer_power_of_two(
                split_point_in_measure.denominator):
                non_power_of_two_factors = mathtools.remove_powers_of_two(
                    split_point_in_measure.denominator)
                non_power_of_two_factors = mathtools.factors(
                    non_power_of_two_factors)
                non_power_of_two_product = 1
                for non_power_of_two_factor in non_power_of_two_factors:
                    non_power_of_two_product *= non_power_of_two_factor
                scoretools.scale_measure_denominator_and_adjust_measure_contents(
                    measure, non_power_of_two_product)
                # rederive duration crosses with possibly new measure contents
                cross_offset = self._get_timespan().start_offset + duration
                duration_crossing_descendants = []
                for descendant in self._get_descendants():
                    start_offset = descendant._get_timespan().start_offset
                    stop_offset = descendant._get_timespan().stop_offset
                    if start_offset < cross_offset < stop_offset:
                        duration_crossing_descendants.append(descendant)
        elif 1 < len(measures):
            message = 'measures can not nest.'
            raise Exception(message)
        # any duration-crossing leaf will be at end of list
        bottom = duration_crossing_descendants[-1]
        did_split_leaf = False
        # if split point necessitates leaf split
        if isinstance(bottom, scoretools.Leaf):
            assert isinstance(bottom, scoretools.Leaf)
            did_split_leaf = True
            split_point_in_bottom = \
                global_split_point - bottom._get_timespan().start_offset
            left_list, right_list = bottom._split_by_duration(
                split_point_in_bottom,
                fracture_spanners=fracture_spanners,
                tie_split_notes=tie_split_notes,
                use_messiaen_style_ties=use_messiaen_style_ties,
                )
            right = right_list[0]
            leaf_right_of_split = right
            leaf_left_of_split = left_list[-1]
            duration_crossing_containers = duration_crossing_descendants[:-1]
            if not len(duration_crossing_containers):
                return left_list, right_list
        # if split point falls between leaves
        # then find leaf to immediate right of split point
        # in order to start upward crawl through duration-crossing containers
        else:
            duration_crossing_containers = duration_crossing_descendants[:]
            for leaf in iterate(bottom).by_leaf():
                if leaf._get_timespan().start_offset == global_split_point:
                    leaf_right_of_split = leaf
                    leaf_left_of_split = leaf_right_of_split._get_leaf(-1)
                    break
            else:
                message = 'can not split empty container {!r}.'
                message = message.format(bottom)
                raise Exception(message)
        # find component to right of split that is also immediate child of
        # last duration-crossing container
        for component in \
            leaf_right_of_split._get_parentage(include_self=True):
            if component._parent is duration_crossing_containers[-1]:
                highest_level_component_right_of_split = component
                break
        else:
            message = 'should we be able to get here?'
            raise ValueError(message)
        # crawl back up through duration-crossing containers and
        # fracture spanners if requested
        if fracture_spanners:
            start_offset = leaf_right_of_split._get_timespan().start_offset
            for parent in leaf_right_of_split._get_parentage():
                if parent._get_timespan().start_offset == start_offset:
                    for spanner in parent._get_spanners():
                        index = spanner._index(parent)
                        spanner._fracture(index, direction=Left)
                if parent is component:
                    break
        # crawl back up through duration-crossing containers and split each
        previous = highest_level_component_right_of_split
        for duration_crossing_container in \
            reversed(duration_crossing_containers):
            assert isinstance(
                duration_crossing_container, scoretools.Container)
            i = duration_crossing_container.index(previous)
            left, right = duration_crossing_container._split_at_index(
                i,
                fracture_spanners=fracture_spanners,
                )
            previous = right
        # NOTE: If logical tie here is convenience, then fusing is good.
        #       If logical tie here is user-given, then fusing is less good.
        #       Maybe later model difference between user logical ties and not.
        left_logical_tie = leaf_left_of_split._get_logical_tie()
        right_logical_tie = leaf_right_of_split._get_logical_tie()
        left_logical_tie._fuse_leaves_by_immediate_parent()
        right_logical_tie._fuse_leaves_by_immediate_parent()
        # reapply tie here if crawl above killed tie applied to leaves
        if did_split_leaf:
            if (
                tie_split_notes and
                isinstance(leaf_left_of_split, scoretools.Note)
                ):
                if (
                    leaf_left_of_split._get_parentage().root is
                    leaf_right_of_split._get_parentage().root
                    ):
                    leaves_around_split = \
                        (leaf_left_of_split, leaf_right_of_split)
                    selection = selectiontools.Selection(
                        leaves_around_split)
                    selection._attach_tie_spanner_to_leaf_pair(
                        use_messiaen_style_ties=use_messiaen_style_ties,
                        )
        # return pair of left and right list-wrapped halves of container
        return ([left], [right])

    ### PUBLIC METHODS ###

    def append(self, component):
        r'''Appends `component` to container.

        ..  container:: example

            Appends note to container:

            ::

                >>> container = Container("c'4 ( d'4 f'4 )")
                >>> show(container) # doctest: +SKIP

            ..  doctest::

                >>> print(format(container))
                {
                    c'4 (
                    d'4
                    f'4 )
                }

            ::

                >>> container.append(Note("e'4"))
                >>> show(container) # doctest: +SKIP

            ..  doctest::

                >>> print(format(container))
                {
                    c'4 (
                    d'4
                    f'4 )
                    e'4
                }

        Returns none.
        '''
        self.__setitem__(slice(len(self), len(self)), [component])

    def extend(self, argument):
        r'''Extends container with `argument`.

        ..  container:: example

            Extends container with three notes:

            ::

                >>> container = Container("c'4 ( d'4 f'4 )")
                >>> show(container) # doctest: +SKIP

            ..  doctest::

                >>> print(format(container))
                {
                    c'4 (
                    d'4
                    f'4 )
                }

            ::

                >>> notes = [Note("e'32"), Note("d'32"), Note("e'16")]
                >>> container.extend(notes)
                >>> show(container) # doctest: +SKIP

            ..  doctest::

                >>> print(format(container))
                {
                    c'4 (
                    d'4
                    f'4 )
                    e'32
                    d'32
                    e'16
                }

        Returns none.
        '''
        self.__setitem__(
            slice(len(self), len(self)),
            argument.__getitem__(slice(0, len(argument)))
            )

    def index(self, component):
        r'''Returns index of `component` in container.

        ..  container:: example

            Gets index of last element in container:

            ::

                >>> container = Container("c'4 d'4 f'4 e'4")
                >>> show(container) # doctest: +SKIP

            ..  doctest::

                >>> print(format(container))
                {
                    c'4
                    d'4
                    f'4
                    e'4
                }

            ::

                >>> note = container[-1]
                >>> note
                Note("e'4")

            ::

                >>> container.index(note)
                3

        Returns nonnegative integer.
        '''
        for i, element in enumerate(self._music):
            if element is component:
                return i
        else:
            message = 'component {!r} not in Abjad container {!r}.'
            message = message.format(component, self)
            raise ValueError(message)

    def insert(self, i, component, fracture_spanners=False):
        r'''Inserts `component` at index `i` in container.

        ..  container:: example

            Inserts note. Does not fracture spanners:

            ::

                >>> container = Container([])
                >>> container.extend("fs16 cs' e' a'")
                >>> container.extend("cs''16 e'' cs'' a'")
                >>> container.extend("fs'16 e' cs' fs")
                >>> slur = spannertools.Slur(direction=Down)
                >>> attach(slur, container[:])
                >>> show(container) # doctest: +SKIP

            ..  doctest::

                >>> print(format(container))
                {
                    fs16 _ (
                    cs'16
                    e'16
                    a'16
                    cs''16
                    e''16
                    cs''16
                    a'16
                    fs'16
                    e'16
                    cs'16
                    fs16 )
                }

            ::

                >>> container.insert(-4, Note("e'4"), fracture_spanners=False)
                >>> show(container) # doctest: +SKIP

            ..  doctest::

                >>> print(format(container))
                {
                    fs16 _ (
                    cs'16
                    e'16
                    a'16
                    cs''16
                    e''16
                    cs''16
                    a'16
                    e'4
                    fs'16
                    e'16
                    cs'16
                    fs16 )
                }

        ..  container:: example

            Inserts note. Fractures spanners:

            ::

                >>> container = Container([])
                >>> container.extend("fs16 cs' e' a'")
                >>> container.extend("cs''16 e'' cs'' a'")
                >>> container.extend("fs'16 e' cs' fs")
                >>> slur = spannertools.Slur(direction=Down)
                >>> attach(slur, container[:])
                >>> show(container) # doctest: +SKIP

            ..  doctest::

                >>> print(format(container))
                {
                    fs16 _ (
                    cs'16
                    e'16
                    a'16
                    cs''16
                    e''16
                    cs''16
                    a'16
                    fs'16
                    e'16
                    cs'16
                    fs16 )
                }

            ::

                >>> container.insert(-4, Note("e'4"), fracture_spanners=True)
                >>> show(container) # doctest: +SKIP

            ..  doctest::

                >>> print(format(container))
                {
                    fs16 _ (
                    cs'16
                    e'16
                    a'16
                    cs''16
                    e''16
                    cs''16
                    a'16 )
                    e'4
                    fs'16 _ (
                    e'16
                    cs'16
                    fs16 )
                }

        Returns none.
        '''
        from abjad.tools import scoretools
        assert isinstance(i, int)
        if not fracture_spanners:
            self.__setitem__(slice(i, i), [component])
            return
        assert isinstance(component, scoretools.Component)
        component._set_parent(self)
        self._music.insert(i, component)
        previous_leaf = component._get_leaf(-1)
        if previous_leaf:
            for spanner in previous_leaf._get_spanners():
                index = spanner._index(previous_leaf)
                spanner._fracture(index, direction=Right)
        next_leaf = component._get_leaf(1)
        if next_leaf:
            for spanner in next_leaf._get_spanners():
                index = spanner._index(next_leaf)
                spanner._fracture(index, direction=Left)

    def pop(self, i=-1):
        r'''Pops component from container at index `i`.

        ..  container:: example

            Pops last element from container:

            ::

                >>> container = Container("c'4 ( d'4 f'4 ) e'4")
                >>> show(container) # doctest: +SKIP

            ..  doctest::

                >>> print(format(container))
                {
                    c'4 (
                    d'4
                    f'4 )
                    e'4
                }

            ::

                >>> container.pop()
                Note("e'4")
                >>> show(container) # doctest: +SKIP

            ..  doctest::

                >>> print(format(container))
                {
                    c'4 (
                    d'4
                    f'4 )
                }

        Returns component.
        '''
        component = self[i]
        del(self[i])
        return component

    def remove(self, component):
        r'''Removes `component` from container.

        ..  container:: example

            Removes note from container:

            ::

                >>> container = Container("c'4 ( d'4 f'4 ) e'4")
                >>> show(container) # doctest: +SKIP

            ..  doctest::

                >>> print(format(container))
                {
                    c'4 (
                    d'4
                    f'4 )
                    e'4
                }

            ::

                >>> note = container[2]
                >>> note
                Note("f'4")

            ::

                >>> container.remove(note)
                >>> show(container) # doctest: +SKIP

            ..  doctest::

                >>> print(format(container))
                {
                    c'4 (
                    d'4 )
                    e'4
                }

        Returns none.
        '''
        i = self.index(component)
        del(self[i])

    def reverse(self):
        r'''Reverses contents of container.

        ..  container:: example

            Reverses staff:

            ::

                >>> staff = Staff("c'8 [ d'8 ] e'8 ( f'8 )")
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> print(format(staff))
                \new Staff {
                    c'8 [
                    d'8 ]
                    e'8 (
                    f'8 )
                }

            ::

                >>> staff.reverse()
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> print(format(staff)) # doctest: +SKIP
                \new Staff {
                    f'8 (
                    e'8 )
                    d'8 [
                    c'8 ]
                }

        Returns none.
        '''
        self._music.reverse()
        self._update_later(offsets=True)
        spanners = self._get_descendants()._get_spanners()
        for s in spanners:
            s._components.sort(key=lambda x: x._get_timespan().start_offset)

    ### PRIVATE PROPERTIES ###

    @property
    def _compact_representation(self):
        if not self:
            return '{ }'
        return '{{ {} }}'.format(self._contents_summary)

    @property
    def _contents_duration(self):
        if self.is_simultaneous:
            return max([durationtools.Duration(0)] +
                [x._preprolated_duration for x in self])
        else:
            duration = durationtools.Duration(0)
            for x in self:
                duration += x._preprolated_duration
            return duration

    @property
    def _contents_summary(self):
        if 0 < len(self):
            result = []
            for x in self._music:
                if hasattr(x, '_compact_representation_with_tie'):
                    result.append(x._compact_representation_with_tie)
                elif hasattr(x, '_compact_representation'):
                    result.append(x._compact_representation)
                else:
                    result.append(str(x))
            return ' '.join(result)
        else:
            return ''

    @property
    def _duration_in_seconds(self):
        if self.is_simultaneous:
            return max([durationtools.Duration(0)] +
                [x._get_duration(in_seconds=True) for x in self])
        else:
            duration = durationtools.Duration(0)
            for leaf in iterate(self).by_leaf():
                duration += leaf._get_duration(in_seconds=True)
            return duration

    @property
    def _preprolated_duration(self):
        return self._contents_duration

    ### PUBLIC PROPERTIES ###

    @property
    def is_simultaneous(self):
        r'''Is true when container is simultaneous. Otherwise false.

        ..  container:: example

            Gets simultaneity status of container:

            ::

                >>> container = Container()
                >>> container.append(Voice("c'8 d'8 e'8"))
                >>> container.append(Voice('g4.'))
                >>> show(container) # doctest: +SKIP

            ..  doctest::

                >>> print(format(container))
                {
                    \new Voice {
                        c'8
                        d'8
                        e'8
                    }
                    \new Voice {
                        g4.
                    }
                }

            ::

                >>> container.is_simultaneous
                False

        ..  container:: example

            Sets simultaneity status of container:

            ::

                >>> container = Container()
                >>> container.append(Voice("c'8 d'8 e'8"))
                >>> container.append(Voice('g4.'))
                >>> show(container) # doctest: +SKIP

            ..  doctest::

                >>> print(format(container))
                {
                    \new Voice {
                        c'8
                        d'8
                        e'8
                    }
                    \new Voice {
                        g4.
                    }
                }

            ::

                >>> container.is_simultaneous = True
                >>> show(container) # doctest: +SKIP

            ..  doctest::

                >>> print(format(container))
                <<
                    \new Voice {
                        c'8
                        d'8
                        e'8
                    }
                    \new Voice {
                        g4.
                    }
                >>

        Defaults to false.

        Set to true or false.

        Returns true or false.
        '''
        return self._is_simultaneous

    @is_simultaneous.setter
    def is_simultaneous(self, argument):
        from abjad.tools import scoretools
        if argument is None:
            return
        assert isinstance(argument, bool), repr(argument)
        prototype = scoretools.Context
        if argument and not all(isinstance(x, prototype) for x in self):
            message = 'simultaneous containers must contain only contexts.'
            raise ValueError(message)
        self._is_simultaneous = argument
        self._update_later(offsets=True)

    @property
    def name(self):
        r'''Gets and sets name of container.

        ..  container:: example

            Gets container name:

            ::

                >>> container = Container("c'4 d'4 e'4 f'4")
                >>> show(container) # doctest: +SKIP

            ..  doctest::

                >>> f(container)
                {
                    c'4
                    d'4
                    e'4
                    f'4
                }

            ::

                >>> container.name is None
                True

        ..  container:: example

            Sets container name:

            ::

                >>> container = Container("c'4 d'4 e'4 f'4", name='Special')
                >>> show(container) # doctest: +SKIP

            ::

                >>> container.name
                'Special'

            Container name does not appear in LilyPond output:

            ::

                >>> f(container)
                {
                    c'4
                    d'4
                    e'4
                    f'4
                }

        Defaults to none.

        Set to string or none.

        Returns string or none.
        '''
        return Component.name.fget(self)

    @name.setter
    def name(self, arg):
        return Component.name.fset(self, arg)



# -*- coding: utf-8 -*-
from abjad import Fraction
from abjad.tools import durationtools
from abjad.tools import selectiontools


def make_repeated_notes_with_shorter_notes_at_end(
    pitch,
    written_duration,
    total_duration,
    prolation=1,
    ):
    r'''Makes repeated notes with `pitch` and `written_duration` summing to
    `total_duration` under `prolation`.

    ::

        >>> arguments = [0, Duration(1, 16), Duration(1, 4)]
        >>> notes = scoretools.make_repeated_notes_with_shorter_notes_at_end(*arguments)
        >>> voice = Voice(notes)

    ..  doctest::

        >>> print(format(voice))
        \new Voice {
            c'16
            c'16
            c'16
            c'16
        }

    Fill power-of-two remaining duration with power-of-two notes of lesser written duration:

    ::

        >>> arguments = [0, Duration(1, 16), Duration(9, 32)]
        >>> notes = scoretools.make_repeated_notes_with_shorter_notes_at_end(*arguments)
        >>> voice = Voice(notes)

    ..  doctest::

        >>> print(format(voice))
        \new Voice {
            c'16
            c'16
            c'16
            c'16
            c'32
        }

    Fill non-power-of-two remaining duration with ad hoc tuplet:

    ::

        >>> arguments = [0, Duration(1, 16), Duration(4, 10)]
        >>> notes = scoretools.make_repeated_notes_with_shorter_notes_at_end(*arguments)
        >>> voice = Voice(notes)

    ..  doctest::

        >>> print(format(voice))
        \new Voice {
            c'16
            c'16
            c'16
            c'16
            c'16
            c'16
            \tweak edge-height #'(0.7 . 0)
            \times 4/5 {
                c'32
            }
        }

    Set `prolation` when making notes in a measure with a non-power-of-two denominator.

    Returns list of components.
    '''
    from abjad.tools import scoretools

    written_duration = durationtools.Duration(written_duration)
    total_duration = durationtools.Duration(total_duration)
    prolation = durationtools.Duration(prolation)
    prolation = Fraction(prolation)

    duration = prolation * written_duration
    current_duration = durationtools.Duration(0)
    result = []
    while current_duration + duration <= total_duration:
        result.append(scoretools.Note(pitch, written_duration))
        current_duration += duration
    remainder_duration = total_duration - current_duration
    if durationtools.Duration(0) < remainder_duration:
        multiplied_remainder = remainder_duration / prolation
        result.extend(scoretools.make_notes(pitch, [multiplied_remainder]))

    result = selectiontools.Selection(result)
    return result



# -*- coding: utf-8 -*-


def yield_all_classes(
    code_root=None,
    include_private_objects=False,
    root_package_name=None,
    ):
    r'''Yields all classes encountered in `code_root`.

    Returns generator.
    '''
    from abjad.tools import documentationtools
    for module in documentationtools.yield_all_modules(
        code_root=code_root,
        root_package_name=root_package_name,
        ):
        name = module.__name__.split('.')[-1]
        if not include_private_objects and name.startswith('_'):
            continue
        if not hasattr(module, name):
            continue
        obj = getattr(module, name)
        if isinstance(obj, type):
            yield obj



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_rhythmmakertools_TaleaRhythmMaker_read_talea_once_only_01():

    maker = rhythmmakertools.TaleaRhythmMaker(
        read_talea_once_only=True,
        talea=rhythmmakertools.Talea(
            counts=[1, 2, 3, 4],
            denominator=16,
            ),
        )

    divisions = [(3, 8), (3, 8), (3, 8), (3, 8)]
    string = 'maker(divisions)'
    assert pytest.raises(Exception, string)



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_pitchtools_NamedPitch___cmp___01():
    r'''Referentially equal pitches compare equally.
    '''

    pitch_1 = NamedPitch('fs', 4)
    assert      pitch_1 == pitch_1
    assert not pitch_1 != pitch_1
    assert not pitch_1 >  pitch_1
    assert      pitch_1 >= pitch_1
    assert not pitch_1 <  pitch_1
    assert      pitch_1 <= pitch_1


def test_pitchtools_NamedPitch___cmp___02():
    r'''Pitches equal by name, accidental and octave compare equally.
    '''

    pitch_1, pitch_2 = NamedPitch('fs', 4), NamedPitch('fs', 4)
    assert      pitch_1 == pitch_2
    assert not pitch_1 != pitch_2
    assert not pitch_1 >  pitch_1
    assert      pitch_1 >= pitch_1
    assert not pitch_1 <  pitch_1
    assert      pitch_1 <= pitch_1


def test_pitchtools_NamedPitch___cmp___03():
    r'''Pitches enharmonically equal compare unequally.
    '''

    pitch_1, pitch_2 = NamedPitch('fs', 4), NamedPitch('gf', 4)
    assert not pitch_1 == pitch_2
    assert      pitch_1 != pitch_2
    assert not pitch_1 >  pitch_2
    assert not pitch_1 >= pitch_2
    assert      pitch_1 <  pitch_2
    assert      pitch_1 <= pitch_2


def test_pitchtools_NamedPitch___cmp___04():
    r'''Pitches manifestly different compare unequally.
    '''

    pitch_1, pitch_2 = NamedPitch('f', 4), NamedPitch('g', 4)
    assert not pitch_1 == pitch_2
    assert      pitch_1 != pitch_2
    assert not pitch_1 >  pitch_2
    assert not pitch_1 >= pitch_2
    assert      pitch_1 <  pitch_2
    assert      pitch_1 <= pitch_2


def test_pitchtools_NamedPitch___cmp___05():
    r'''Pitches typographically crossed compare unequally.
    '''

    pitch_1, pitch_2 = NamedPitch('fss', 4), NamedPitch('gff', 4)
    assert not pitch_1 == pitch_2
    assert      pitch_1 != pitch_2
    assert not pitch_1 >  pitch_2
    assert not pitch_1 >= pitch_2
    assert      pitch_1 <  pitch_2
    assert      pitch_1 <= pitch_2


def test_pitchtools_NamedPitch___cmp___06():
    r'''Pitches test false for equality against unlike instances.
    Other pitch comparisons raise ValueError against unlike instances.
    '''

    pitch = NamedPitch('c', 4)
    number = 99
    assert not pitch == number
    assert      pitch != number



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_scoretools_make_repeated_notes_with_shorter_notes_at_end_01():
    r'''Makes repeated 1/16th notes equal to 1/4 total duration.
    '''

    voice = Voice(scoretools.make_repeated_notes_with_shorter_notes_at_end(
        0, Duration(1, 16), Duration(1, 4)))

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'16
            c'16
            c'16
            c'16
        }
        '''
        )

    assert inspect_(voice).is_well_formed()


def test_scoretools_make_repeated_notes_with_shorter_notes_at_end_02():
    r'''Makes repeated 1/16th notes equal to 9/32 total duration.
    '''

    voice = Voice(scoretools.make_repeated_notes_with_shorter_notes_at_end(
        0, Duration(1, 16), Duration(9, 32)))

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'16
            c'16
            c'16
            c'16
            c'32
        }
        '''
        )

    assert inspect_(voice).is_well_formed()


def test_scoretools_make_repeated_notes_with_shorter_notes_at_end_03():
    r'''Makes repeated 1/16th notes equal to only 1/128 total duration.
    '''

    voice = Voice(scoretools.make_repeated_notes_with_shorter_notes_at_end(
        0, Duration(1, 16), Duration(1, 128)))

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'128
        }
        '''
        )

    assert inspect_(voice).is_well_formed()


def test_scoretools_make_repeated_notes_with_shorter_notes_at_end_04():
    r'''Makes repeated 1/16th notes equal to 4/10 total duration.
    '''

    voice = Voice(scoretools.make_repeated_notes_with_shorter_notes_at_end(
        0, Duration(1, 16), Duration(4, 10)))

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'16
            c'16
            c'16
            c'16
            c'16
            c'16
            \tweak edge-height #'(0.7 . 0)
            \times 4/5 {
                c'32
            }
        }
        '''
        )

    assert inspect_(voice).is_well_formed()


def test_scoretools_make_repeated_notes_with_shorter_notes_at_end_05():
    r'''Makes repeated written 1/16th notes within measure of 5/18.
    '''

    measure = Measure(
        (5, 18),
        scoretools.make_repeated_notes_with_shorter_notes_at_end(
            0,
            Duration(1, 16),
            Duration(5, 18),
            prolation=Duration(16, 18)),
        implicit_scaling=True,
        )

    assert format(measure) == stringtools.normalize(
        r'''
        {
            \time 5/18
            \scaleDurations #'(8 . 9) {
                c'16
                c'16
                c'16
                c'16
                c'16
            }
        }
        '''
        )

    assert inspect_(measure).is_well_formed()



# -*- coding: utf-8 -*-
import copy
import inspect
import pytest
from abjad.tools import abjadbooktools
from abjad.tools import documentationtools
from abjad.tools import lilypondparsertools
from abjad.tools import quantizationtools
from abjad.tools import rhythmtreetools
from abjad.tools import scoretools
from abjad.tools import systemtools
from abjad.tools import tonalanalysistools


ignored_classes = (
    abjadbooktools.CodeBlock,
    abjadbooktools.CodeOutputProxy,
    abjadbooktools.GraphvizOutputProxy,
    abjadbooktools.LilyPondOutputProxy,
    lilypondparsertools.LilyPondParser,
    lilypondparsertools.ReducedLyParser,
    lilypondparsertools.SchemeParser,
    quantizationtools.ParallelJobHandlerWorker,
    rhythmtreetools.RhythmTreeParser,
    systemtools.RedirectedStreams,
    systemtools.StorageFormatAgent,
    systemtools.FormatSpecification,
    tonalanalysistools.RootedChordClass
    )

classes = documentationtools.list_all_abjad_classes(
    ignored_classes=ignored_classes,
    )


@pytest.mark.parametrize('class_', classes)
def test_abjad___deepcopy___01(class_):
    r'''All concrete classes with a storage format can copy.
    '''
    if (
        '_get_storage_format_specification' not in dir(class_) or
        '_get_format_specification' not in dir(class_)
        ):
        return
    if inspect.isabstract(class_):
        return
    instance_one = class_()
    instance_two = copy.deepcopy(instance_one)
    instance_one_format = format(instance_one, 'storage')
    instance_two_format = format(instance_two, 'storage')
    if not issubclass(class_, scoretools.Container):
        assert instance_one_format == instance_two_format
    # TODO: eventually this second asset should also pass
    #assert instance_one == instance_two



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_NamedInterval_named_interval_class_01():

    named_interval = pitchtools.NamedInterval('perfect', 1)
    #assert pitchtools.NamedIntervalClass(named_interval) == 1
    ic = pitchtools.NamedIntervalClass(named_interval)
    assert ic == pitchtools.NamedIntervalClass('perfect', 1)

    named_interval = pitchtools.NamedInterval('minor', 2)
    #assert pitchtools.NamedIntervalClass(named_interval) == 2
    ic = pitchtools.NamedIntervalClass(named_interval)
    assert ic == pitchtools.NamedIntervalClass('minor', 2)

    named_interval = pitchtools.NamedInterval('major', 2)
    #assert pitchtools.NamedIntervalClass(named_interval) == 2
    ic = pitchtools.NamedIntervalClass(named_interval)
    assert ic == pitchtools.NamedIntervalClass('major', 2)

    named_interval = pitchtools.NamedInterval('minor', 3)
    #assert pitchtools.NamedIntervalClass(named_interval) == 3
    ic = pitchtools.NamedIntervalClass(named_interval)
    assert ic == pitchtools.NamedIntervalClass('minor', 3)

    named_interval = pitchtools.NamedInterval('major', 3)
    #assert pitchtools.NamedIntervalClass(named_interval) == 3
    ic = pitchtools.NamedIntervalClass(named_interval)
    assert ic == pitchtools.NamedIntervalClass('major', 3)


def test_pitchtools_NamedInterval_named_interval_class_02():

    named_interval = pitchtools.NamedInterval('perfect', 8)
    #assert pitchtools.NamedIntervalClass(named_interval) == 1
    ic = pitchtools.NamedIntervalClass(named_interval)
    assert ic == pitchtools.NamedIntervalClass('perfect', 8)

    named_interval = pitchtools.NamedInterval('minor', 9)
    #assert pitchtools.NamedIntervalClass(named_interval) == 2
    ic = pitchtools.NamedIntervalClass(named_interval)
    assert ic == pitchtools.NamedIntervalClass('minor', 2)

    named_interval = pitchtools.NamedInterval('major', 9)
    #assert pitchtools.NamedIntervalClass(named_interval) == 2
    ic = pitchtools.NamedIntervalClass(named_interval)
    assert ic == pitchtools.NamedIntervalClass('major', 2)

    named_interval = pitchtools.NamedInterval('minor', 10)
    #assert pitchtools.NamedIntervalClass(named_interval) == 3
    ic = pitchtools.NamedIntervalClass(named_interval)
    assert ic == pitchtools.NamedIntervalClass('minor', 3)

    named_interval = pitchtools.NamedInterval('major', 10)
    #assert pitchtools.NamedIntervalClass(named_interval) == 3
    ic = pitchtools.NamedIntervalClass(named_interval)
    assert ic == pitchtools.NamedIntervalClass('major', 3)


def test_pitchtools_NamedInterval_named_interval_class_03():

    named_interval = pitchtools.NamedInterval('perfect', -8)
    #assert pitchtools.NamedIntervalClass(named_interval) == -1
    ic = pitchtools.NamedIntervalClass(named_interval)
    assert ic == pitchtools.NamedIntervalClass('perfect', -8)

    named_interval = pitchtools.NamedInterval('minor', -9)
    #assert pitchtools.NamedIntervalClass(named_interval) == -2
    ic = pitchtools.NamedIntervalClass(named_interval)
    assert ic == pitchtools.NamedIntervalClass('minor', -2)

    named_interval = pitchtools.NamedInterval('major', -9)
    #assert pitchtools.NamedIntervalClass(named_interval) == -2
    ic = pitchtools.NamedIntervalClass(named_interval)
    assert ic == pitchtools.NamedIntervalClass('major', -2)

    named_interval = pitchtools.NamedInterval('minor', -10)
    #assert pitchtools.NamedIntervalClass(named_interval) == -3
    ic = pitchtools.NamedIntervalClass(named_interval)
    assert ic == pitchtools.NamedIntervalClass('minor', -3)

    named_interval = pitchtools.NamedInterval('major', -10)
    #assert pitchtools.NamedIntervalClass(named_interval) == -3
    ic = pitchtools.NamedIntervalClass(named_interval)
    assert ic == pitchtools.NamedIntervalClass('major', -3)



# -*- coding: utf-8 -*-
import multiprocessing
import pickle
from abjad.tools.abctools import AbjadObject


class ParallelJobHandlerWorker(multiprocessing.Process, AbjadObject):
    r'''Worker process which runs ``QuantizationJobs``.

    Not composer-safe.

    Used internally by ``ParallelJobHandler``.
    '''

    ### INITIALIZER ###

    def __init__(self, job_queue=None, result_queue=None):
        multiprocessing.Process.__init__(self)
        job_queue = job_queue or ()
        result_queue = result_queue or ()
        self.job_queue = job_queue
        self.result_queue = result_queue

    ### PUBLIC METHODS ###

    def run(self):
        r'''Runs parallel job handler worker.

        Returns none.
        '''
        process_name = self.name
        while True:
            job = self.job_queue.get()
            if job is None:
                # poison pill causes worker shutdown
                #print '{}: Exiting'.format(process_name)
                self.job_queue.task_done()
                break
            #print '{}: {!r}'.format(process_name, job)
            job = pickle.loads(job)
            job()
            self.job_queue.task_done()
            self.result_queue.put(pickle.dumps(job, protocol=0))
        return



# -*- coding: utf-8 -*-
import functools
from abjad.tools import mathtools
from abjad.tools.pitchtools.Pitch import Pitch


@functools.total_ordering
class NumberedPitch(Pitch):
    '''Numbered pitch.

    ..  container:: example

        Initializes from number:

        ::

            >>> numbered_pitch = NumberedPitch(13)
            >>> show(numbered_pitch) # doctest: +SKIP

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_pitch_number',
        )

    ### INITIALIZER ###

    def __init__(self, pitch_number=None):
        from abjad.tools import pitchtools
        if hasattr(pitch_number, 'pitch_number'):
            pitch_number = pitch_number.pitch_number
        elif Pitch.is_pitch_number(pitch_number):
            pitch_number = pitch_number
        elif pitch_number is None:
            pitch_number = 0
        else:
            pitch_number = pitchtools.NamedPitch(pitch_number).pitch_number
        pitch_number = mathtools.integer_equivalent_number_to_integer(
            pitch_number)
        self._pitch_number = pitch_number

    ### SPECIAL METHODS ###

    def __add__(self, argument):
        r'''Adds `argument` to numberd pitch.

        Returns new numbered pitch.
        '''
        argument = type(self)(argument)
        semitones = self.pitch_number + argument.pitch_number
        return type(self)(semitones)

    def __float__(self):
        r'''Changes numbered pitch to float.

        Returns float.
        '''
        return float(self._pitch_number)

    def __int__(self):
        r'''Changes numbered pitch to integer.

        Returns integer.
        '''
        return self._pitch_number

    def __lt__(self, argument):
        r'''Is true when `argument` can be coerced to a numbered pitch and when this
        numbered pitch is less than `argument`. Otherwise false.

        Returns true or false.
        '''
        try:
            argument = type(self)(argument)
        except (ValueError, TypeError):
            return False
        return self.pitch_number < argument.pitch_number

    def __neg__(self):
        r'''Negates numbered pitch.

        Returns new numbered pitch.
        '''
        return type(self)(-self.pitch_number)

    def __str__(self):
        r'''String representation of numbered pitch.

        Returns string.
        '''
        return str(self.pitch_number)

    def __sub__(self, argument):
        r'''Subtracts `argument` from numbered pitch.

        Returns numbered interval.
        '''
        from abjad.tools import pitchtools
        if isinstance(argument, type(self)):
            return pitchtools.NumberedInterval.from_pitch_carriers(
                self, argument)
        interval = pitchtools.NumberedInterval(argument)
        interval = -interval
        return interval.transpose(self)

    ### PRIVATE METHODS ###

    def _get_lilypond_format(self):
        return self.pitch_name

    ### PUBLIC PROPERTIES ###

    @property
    def accidental(self):
        r'''Gets accidental of numbered pitch.

        ..  container:: example

            ::

                >>> NumberedPitchClass(13).accidental
                Accidental('s')

        Returns accidental.
        '''
        from abjad.tools import pitchtools
        return pitchtools.Accidental(self.alteration_in_semitones)

    @property
    def alteration_in_semitones(self):
        r'''Gets alteration of numbered pitch in semitones.

        ..  container:: example

            ::

                >>> NumberedPitchClass(13).alteration_in_semitones
                1

        Returns integer or float.
        '''
        return self.numbered_pitch_class.alteration_in_semitones

    @property
    def diatonic_pitch_class_name(self):
        r'''Gets diatonic pitch-class name corresponding to numbered pitch.

        ..  note:: Deprecated.

        ..  container:: example

            ::

                >>> NumberedPitch(13).diatonic_pitch_class_name
                'c'

        Returns string.
        '''
        return self.numbered_pitch_class.diatonic_pitch_class_name

    @property
    def diatonic_pitch_class_number(self):
        r'''Gets diatonic pitch-class number of numbered pitch.

        ..  note:: Deprecated.

        ..  container:: example

            ::

                >>> NumberedPitch(13).diatonic_pitch_class_number
                0

        Returns integer.
        '''
        return self.numbered_pitch_class.diatonic_pitch_class_number

    @property
    def diatonic_pitch_name(self):
        r'''Gets diatonic pitch name of numbered pitch.

        ..  note:: Deprecated.

        ..  container:: example

            ::

                >>> NumberedPitch(13).diatonic_pitch_name
                "c''"

        Returns string.
        '''
        return '{}{}'.format(
            self.diatonic_pitch_class_name,
            self.octave.tick_string,
            )

    @property
    def diatonic_pitch_number(self):
        r'''Gets diatonic pitch-class number corresponding to numbered pitch.

        ..  note:: Deprecated.

        ..  container:: example

            ::

                >>> NumberedPitch(13).diatonic_pitch_number
                7

        Returns integer.
        '''
        return 7 * (self.octave.number - 4) + self.diatonic_pitch_class_number

    @property
    def name(self):
        r'''Gets pitch name.

        ..  container:: example

            ::

                >>> NumberedPitch(13).name
                "cs''"

        Returns string
        '''
        from abjad.tools import pitchtools
        return pitchtools.NamedPitch(self).name

    @property
    def named_pitch(self):
        r'''Gets named pitch corresponding to numbered pitch.

        ..  note:: Deprecated.

        ..  container:: example

            ::

                >>> NumberedPitch(13).named_pitch
                NamedPitch("cs''")

        Returns named pitch.
        '''
        from abjad.tools import pitchtools
        return pitchtools.NamedPitch(self)

    @property
    def named_pitch_class(self):
        r'''Gets named pitch-class corresponding to numbered pitch.

        ..  note:: Deprecated.

        ..  container:: example

            ::

                >>> NumberedPitch(13).named_pitch_class
                NamedPitchClass('cs')

        Returns named pitch-class.
        '''
        from abjad.tools import pitchtools
        return pitchtools.NamedPitchClass(self)

    @property
    def number(self):
        r'''Gets pitch number.

        ..  container:: example

            ::

                >>> NumberedPitch(13).number
                13

        Returns number.
        '''
        return self._pitch_number

    @property
    def numbered_pitch(self):
        r'''Gets numbered pitch.

        ..  note:: Deprecated.

        ..  container:: example

            ::

                >>> NumberedPitch(13).numbered_pitch
                NumberedPitch(13)

        Returns new numbered pitch.
        '''
        return type(self)(self)

    @property
    def numbered_pitch_class(self):
        r'''Gets numbered pitch-class corresponding to numbered pitch.

        ..  note:: Deprecated.

        ..  container:: example

            ::

                >>> NumberedPitch(13).numbered_pitch_class
                NumberedPitchClass(1)

        Returns numbered pitch-class.
        '''
        from abjad.tools import pitchtools
        return pitchtools.NumberedPitchClass(self)

    @property
    def octave(self):
        r'''Gets octave of numbered pitch.

        ..  container:: example

            ::

                >>> NumberedPitch(13).octave
                Octave(5)

        Returns octave.
        '''
        from abjad.tools import pitchtools
        number = self._pitch_number // 12 + 4
        return pitchtools.Octave(number=number)

    @property
    def pitch_class(self):
        r'''Gets pitch-class.

        ..  container:: example

            ::

                >>> NumberedPitch(13).pitch_class
                NumberedPitchClass(1)

        Returns numbered pitch-class.
        '''
        from abjad.tools import pitchtools
        return pitchtools.NumberedPitchClass(self)

    @property
    def pitch_class_name(self):
        r'''Gets pitch-class name of numbered pitch.

        ..  note:: Deprecated.

        ..  container:: example

            ::

                >>> NumberedPitch(13).pitch_class_name
                'cs'

        Returns string.
        '''
        return self.numbered_pitch_class.pitch_class_name

    @property
    def pitch_class_number(self):
        r'''Gets pitch-class number of numbered pitch.

        ..  note:: Deprecated.

        ..  container:: example

            ::

                >>> NumberedPitch(13).pitch_class_number
                1

        Returns integer or float.
        '''
        return self._pitch_number % 12

    @property
    def pitch_class_octave_label(self):
        r'''Gets pitch-class / octave label of numbered pitch.

        ..  note:: Deprecated.

        ..  container:: example

            ::

                >>> NumberedPitch(13).pitch_class_octave_label
                'C#5'

        Returns string.
        '''
        return '{}{}{}'.format(
            self.diatonic_pitch_class_name.upper(),
            self.accidental.symbolic_string,
            self.octave.number,
            )

    @property
    def pitch_name(self):
        r'''Gets pitch name corresponding to numbered pitch.

        ..  note:: Deprecated.

        ..  container:: example

            ::

                >>> NumberedPitch(13).pitch_name
                "cs''"

        Returns string.
        '''
        return '{}{}'.format(
            self.numbered_pitch_class.pitch_class_name,
            self.octave.tick_string,
            )

    @property
    def pitch_number(self):
        r'''Gets pitch number of numbered pitch.

        ..  note:: Deprecated.

        ..  container:: example

            ::

                >>> NumberedPitch(13).pitch_number
                13

        Returns number.
        '''
        return self._pitch_number

    ### PUBLIC METHODS ###

    def apply_accidental(self, accidental=None):
        r'''Applies `accidental` to numbered pitch.

        ..  container:: example

            ::

                >>> NumberedPitch(13).apply_accidental('flat')
                NumberedPitch(12)

        Returns new numbered pitch.
        '''
        from abjad.tools.pitchtools.Accidental import Accidental
        accidental = Accidental(accidental)
        semitones = self.pitch_number + accidental.semitones
        return type(self)(semitones)

    @staticmethod
    def from_pitch_class_octave(pitch_class, octave):
        r'''Initializes numbered pitch from `pitch_class` and `octave`.

        ..  container:: example

            ::

                >>> for octave in [1, 2, 3, 4, 5, 6, 7]:
                ...     NumberedPitch.from_pitch_class_octave(6, octave)
                ...
                NumberedPitch(-30)
                NumberedPitch(-18)
                NumberedPitch(-6)
                NumberedPitch(6)
                NumberedPitch(18)
                NumberedPitch(30)
                NumberedPitch(42)

        Returns new numbered pitch.
        '''
        from abjad.tools import pitchtools
        pitch_class = pitchtools.NumberedPitchClass(pitch_class)
        octave = pitchtools.Octave(octave)
        number = 12 * (octave.number - 4) + pitch_class.number
        return NumberedPitch(number)

    def interpolate(self, stop_pitch, fraction):
        r'''Interpolates between this pitch and `stop_pitch` by `fraction`
        amount.

        ..  container:: example

            Interpolates up from C4 to C5:

            ::

                >>> start_pitch = NumberedPitch(0)
                >>> stop_pitch = NumberedPitch(12)

            ::

                >>> start_pitch.interpolate(stop_pitch, Fraction(0))
                NumberedPitch(0)
                >>> start_pitch.interpolate(stop_pitch, Fraction(1, 4))
                NumberedPitch(3)
                >>> start_pitch.interpolate(stop_pitch, Fraction(1, 2))
                NumberedPitch(6)
                >>> start_pitch.interpolate(stop_pitch, Fraction(3, 4))
                NumberedPitch(9)
                >>> start_pitch.interpolate(stop_pitch, Fraction(1))
                NumberedPitch(12)

        ..  container:: example

            Interpolates down from C5 to C4:

            ::

                >>> start_pitch = NumberedPitch(12)
                >>> stop_pitch = NumberedPitch(0)

            ::

                >>> start_pitch.interpolate(stop_pitch, Fraction(0))
                NumberedPitch(12)
                >>> start_pitch.interpolate(stop_pitch, Fraction(1, 4))
                NumberedPitch(9)
                >>> start_pitch.interpolate(stop_pitch, Fraction(1, 2))
                NumberedPitch(6)
                >>> start_pitch.interpolate(stop_pitch, Fraction(3, 4))
                NumberedPitch(3)
                >>> start_pitch.interpolate(stop_pitch, Fraction(1))
                NumberedPitch(0)

        Returns new pitch.
        '''
        from abjad.tools import pitchtools
        assert 0 <= fraction <= 1, repr(fraction)
        stop_pitch = type(self)(stop_pitch)
        distance = stop_pitch - self
        distance = abs(distance.semitones)
        distance = fraction * distance
        distance = int(distance)
        if stop_pitch < self:
            distance *= -1
        pitch_number = self.pitch_number
        pitch_number = pitch_number + distance
        pitch = pitchtools.NumberedPitch(pitch_number)
        if self <= stop_pitch:
            triple = (self, pitch, stop_pitch)
            assert self <= pitch <= stop_pitch, triple
        else:
            triple = (self, pitch, stop_pitch)
            assert self >= pitch >= stop_pitch, triple
        return pitch

    def invert(self, axis=None):
        r'''Inverts numberd pitch around `axis`.

        ..  container:: example

            Inverts pitch-class about pitch-class 0 explicitly:

            ::

                >>> NumberedPitch(2).invert(0)
                NumberedPitch(-2)

            ::

                >>> NumberedPitch(-2).invert(0)
                NumberedPitch(2)

        ..  container:: example

            Inverts pitch-class about pitch-class 0 implicitly:

            ::

                >>> NumberedPitch(2).invert()
                NumberedPitch(-2)

            ::

                >>> NumberedPitch(-2).invert()
                NumberedPitch(2)

        ..  container:: example

            Inverts pitch-class about pitch-class -3:

            ::

                >>> NumberedPitch(2).invert(-3)
                NumberedPitch(-8)

        Returns new numbered pitch.
        '''
        return Pitch.invert(self, axis=axis)

    def multiply(self, n=1):
        r'''Multiplies pitch by index `n`.

        ..  container:: example

            ::

                >>> NumberedPitch(14).multiply(3)
                NumberedPitch(42)

        Returns new numbered pitch.
        '''
        #pitch_class_number = (self.pitch_class_number * n) % 12
        #octave_floor = (self.octave.number - 4) * 12
        #return type(self)(pitch_class_number + octave_floor)
        return type(self)(n * self.number)

    def transpose(self, n=0):
        r'''Tranposes numbered pitch by `n` semitones.

        ..  container:: example

            ::

                >>> NumberedPitch(13).transpose(1)
                NumberedPitch(14)

        Returns new numbered pitch.
        '''
        semitones = self.pitch_number + float(n)
        return type(self)(semitones)



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_Measure__duration_and_possible_denominators_to_time_signature_01():
    r'''Find only feasible denominator in denominators list.
    '''

    time_signature = \
        Measure._duration_and_possible_denominators_to_time_signature(
        Duration(3, 2),
        [5, 6, 7, 8, 9],
        )

    assert time_signature == TimeSignature((9, 6))


def test_scoretools_Measure__duration_and_possible_denominators_to_time_signature_02():
    r'''Use least feasible denominator in denominators list.
    '''

    time_signature = \
        Measure._duration_and_possible_denominators_to_time_signature(
        Duration(3, 2),
        [4, 8, 16, 32],
        )

    assert time_signature == TimeSignature((6, 4))


def test_scoretools_Measure__duration_and_possible_denominators_to_time_signature_03():
    r'''Make time signature literally from time_signature.
    '''

    time_signature = \
        Measure._duration_and_possible_denominators_to_time_signature(
        Duration(3, 2),
        )

    assert time_signature == TimeSignature((3, 2))


def test_scoretools_Measure__duration_and_possible_denominators_to_time_signature_04():
    r'''Make time signature literally from time_signature
    because no feasible denomiantors in denominators list.
    '''

    time_signature = \
        Measure._duration_and_possible_denominators_to_time_signature(
        Duration(3, 2),
        [7, 11, 13, 19],
        )

    assert time_signature == TimeSignature((3, 2))



# -*- coding: utf-8 -*-
from abjad import *


def test_selectiontools_VerticalMoment___eq___01():

    score = Score(
        r'''
        \new Staff {
            \times 4/3 {
                d''8
                c''8
                b'8
            }
        }
        \new PianoStaff <<
            \new Staff {
                a'4
                g'4
            }
            \new Staff {
                \clef "bass"
                f'8
                e'8
                d'8
                c'8
            }
        >>
        '''
        )

    staff_group = score[1]

    vertical_moment_1 = inspect_(staff_group).get_vertical_moment_at(Offset(1, 8))
    "VerticalMoment(PianoStaff<<2>>, Staff{2}, a'4, Staff{4}, e'8)"

    vertical_moment_2 = inspect_(staff_group).get_vertical_moment_at(Offset(1, 8))
    "VerticalMoment(PianoStaff<<2>>, Staff{2}, a'4, Staff{4}, e'8)"

    assert vertical_moment_1 == vertical_moment_2
    assert not vertical_moment_1 != vertical_moment_2


def test_selectiontools_VerticalMoment___eq___02():

    score = Score([])
    score.append(Staff([scoretools.FixedDurationTuplet(
        Duration(4, 8), "d''8 c''8 b'8")]))
    staff_group = StaffGroup(r'''
        \new Staff {
            a'4
            g'4
        }
        \new Staff {
            \clef "bass"
            f'8
            e'8
            d'8
            c'8
        }
        '''
        )
    staff_group.context_name = 'PianoStaff'
    score.append(staff_group)

    assert format(score) == stringtools.normalize(
        r'''
        \new Score <<
            \new Staff {
                \tweak text #tuplet-number::calc-fraction-text
                \times 4/3 {
                    d''8
                    c''8
                    b'8
                }
            }
            \new PianoStaff <<
                \new Staff {
                    a'4
                    g'4
                }
                \new Staff {
                    \clef "bass"
                    f'8
                    e'8
                    d'8
                    c'8
                }
            >>
        >>
        '''
        )

    vertical_moment_1 = inspect_(
        staff_group).get_vertical_moment_at(Offset(1, 8))
    "VerticalMoment(PianoStaff<<2>>, Staff{2}, a'4, Staff{4}, e'8)"

    vertical_moment_2 = inspect_(
        staff_group[0]).get_vertical_moment_at(Offset(1, 8))
    "VerticalMoment(Staff{2}, a'4, Staff{4}, e'8)"

    assert not vertical_moment_1 == vertical_moment_2
    assert vertical_moment_1 != vertical_moment_2



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_PitchClassSegment_rotate_01():

    pitch_class_segment = pitchtools.PitchClassSegment([0, 6, 10, 4, 9, 2])
    assert pitch_class_segment.rotate(0) == pitchtools.PitchClassSegment(
        [0, 6, 10, 4, 9, 2])


def test_pitchtools_PitchClassSegment_rotate_02():
    r'''Rotate right.
    '''

    pitch_class_segment = pitchtools.PitchClassSegment([0, 6, 10, 4, 9, 2])
    assert pitch_class_segment.rotate(1) == \
        pitchtools.PitchClassSegment([2, 0, 6, 10, 4, 9])
    assert pitch_class_segment.rotate(2) == \
        pitchtools.PitchClassSegment([9, 2, 0, 6, 10, 4])
    assert pitch_class_segment.rotate(3) == \
        pitchtools.PitchClassSegment([4, 9, 2, 0, 6, 10])
    assert pitch_class_segment.rotate(4) == \
        pitchtools.PitchClassSegment([10, 4, 9, 2, 0, 6])
    assert pitch_class_segment.rotate(5) == \
        pitchtools.PitchClassSegment([6, 10, 4, 9, 2, 0])
    assert pitch_class_segment.rotate(6) == \
        pitchtools.PitchClassSegment([0, 6, 10, 4, 9, 2])


def test_pitchtools_PitchClassSegment_rotate_03():
    r'''Rotate left.
    '''

    pitch_class_segment = pitchtools.PitchClassSegment([0, 6, 10, 4, 9, 2])
    assert pitch_class_segment.rotate(-1) == \
        pitchtools.PitchClassSegment([6, 10, 4, 9, 2, 0])
    assert pitch_class_segment.rotate(-2) == \
        pitchtools.PitchClassSegment([10, 4, 9, 2, 0, 6])
    assert pitch_class_segment.rotate(-3) == \
        pitchtools.PitchClassSegment([4, 9, 2, 0, 6, 10])
    assert pitch_class_segment.rotate(-4) == \
        pitchtools.PitchClassSegment([9, 2, 0, 6, 10, 4])
    assert pitch_class_segment.rotate(-5) == \
        pitchtools.PitchClassSegment([2, 0, 6, 10, 4, 9])
    assert pitch_class_segment.rotate(-6) == \
        pitchtools.PitchClassSegment([0, 6, 10, 4, 9, 2])


def test_pitchtools_PitchClassSegment_rotate_04():

    pitch_class_segment_1 = pitchtools.PitchClassSegment([
        pitchtools.NamedPitchClass('c'),
        pitchtools.NamedPitchClass('d'),
        pitchtools.NamedPitchClass('e'),
        pitchtools.NamedPitchClass('f'),
        pitchtools.NamedPitchClass('g'),])
    pitch_class_segment_2 = pitchtools.PitchClassSegment([
        pitchtools.NamedPitchClass('g'),
        pitchtools.NamedPitchClass('c'),
        pitchtools.NamedPitchClass('d'),
        pitchtools.NamedPitchClass('e'),
        pitchtools.NamedPitchClass('f'),])
    assert pitch_class_segment_1.rotate(1) == pitch_class_segment_2
    assert pitch_class_segment_1.rotate(-4) == pitch_class_segment_2
    assert pitch_class_segment_2.rotate(-1) == pitch_class_segment_1
    assert pitch_class_segment_2.rotate(4) == pitch_class_segment_1



# -*- coding: utf-8 -*-
from abjad import *
from abjad.tools import tonalanalysistools
import pytest


def test_tonalanalysistools_Scale_scale_degree_to_named_pitch_class_01():

    scale = tonalanalysistools.Scale('c', 'major')

    assert scale.scale_degree_to_named_pitch_class(1) == \
        pitchtools.NamedPitchClass('c')
    assert scale.scale_degree_to_named_pitch_class(2) == \
        pitchtools.NamedPitchClass('d')
    assert scale.scale_degree_to_named_pitch_class(3) == \
        pitchtools.NamedPitchClass('e')
    assert scale.scale_degree_to_named_pitch_class(4) == \
        pitchtools.NamedPitchClass('f')
    assert scale.scale_degree_to_named_pitch_class(5) == \
        pitchtools.NamedPitchClass('g')
    assert scale.scale_degree_to_named_pitch_class(6) == \
        pitchtools.NamedPitchClass('a')
    assert scale.scale_degree_to_named_pitch_class(7) == \
        pitchtools.NamedPitchClass('b')


def test_tonalanalysistools_Scale_scale_degree_to_named_pitch_class_02():

    scale = tonalanalysistools.Scale('c', 'major')

    assert scale.scale_degree_to_named_pitch_class('flat', 1) == \
        pitchtools.NamedPitchClass('cf')
    assert scale.scale_degree_to_named_pitch_class('flat', 2) == \
        pitchtools.NamedPitchClass('df')
    assert scale.scale_degree_to_named_pitch_class('flat', 3) == \
        pitchtools.NamedPitchClass('ef')
    assert scale.scale_degree_to_named_pitch_class('flat', 4) == \
        pitchtools.NamedPitchClass('ff')
    assert scale.scale_degree_to_named_pitch_class('flat', 5) == \
        pitchtools.NamedPitchClass('gf')
    assert scale.scale_degree_to_named_pitch_class('flat', 6) == \
        pitchtools.NamedPitchClass('af')
    assert scale.scale_degree_to_named_pitch_class('flat', 7) == \
        pitchtools.NamedPitchClass('bf')


def test_tonalanalysistools_Scale_scale_degree_to_named_pitch_class_03():

    scale = tonalanalysistools.Scale('c', 'major')

    assert scale.scale_degree_to_named_pitch_class('sharp', 1) == \
        pitchtools.NamedPitchClass('cs')
    assert scale.scale_degree_to_named_pitch_class('sharp', 2) == \
        pitchtools.NamedPitchClass('ds')
    assert scale.scale_degree_to_named_pitch_class('sharp', 3) == \
        pitchtools.NamedPitchClass('es')
    assert scale.scale_degree_to_named_pitch_class('sharp', 4) == \
        pitchtools.NamedPitchClass('fs')
    assert scale.scale_degree_to_named_pitch_class('sharp', 5) == \
        pitchtools.NamedPitchClass('gs')
    assert scale.scale_degree_to_named_pitch_class('sharp', 6) == \
        pitchtools.NamedPitchClass('as')
    assert scale.scale_degree_to_named_pitch_class('sharp', 7) == \
        pitchtools.NamedPitchClass('bs')


def test_tonalanalysistools_Scale_scale_degree_to_named_pitch_class_04():

    scale = tonalanalysistools.Scale('c', 'major')

    assert pytest.raises(ValueError,
        'scale.scale_degree_to_named_pitch_class(99)')



# -*- coding: utf-8 -*-
from abjad.tools import schemetools
from abjad.tools.topleveltools import override
from abjad.tools.topleveltools import set_


def configure_score(score):
    r'''Configured score.
    '''

    moment = schemetools.SchemeMoment(1, 56)
    set_(score).proportional_notation_duration = moment
    set_(score).tuplet_full_length = True
    override(score).bar_line.stencil = False
    override(score).bar_number.transparent = True
    override(score).spacing_spanner.uniform_stretching = True
    override(score).spacing_spanner.strict_note_spacing = True
    override(score).time_signature.stencil = False
    override(score).tuplet_bracket.padding = 2
    override(score).tuplet_bracket.staff_padding = 4
    scheme = schemetools.Scheme('tuplet-number::calc-fraction-text')
    override(score).tuplet_number.text = scheme


# -*- coding: utf-8 -*-
from abjad import *


def test_tonalanalysistools_ChordQuality___eq___01():

    chord_quality = tonalanalysistools.ChordQuality('major')
    u = tonalanalysistools.ChordQuality('major')
    voice = tonalanalysistools.ChordQuality('minor')

    assert     chord_quality == chord_quality
    assert     chord_quality == u
    assert not chord_quality == voice

    assert     u == chord_quality
    assert     u == u
    assert not u == voice

    assert not voice == chord_quality
    assert not voice == u
    assert     voice == voice



from abjad import *


def test_scoretools_Tuplet_toggle_prolation_01():
    '''Change augmentation to diminution.
    '''

    tuplet = Tuplet((4, 3), "c'8 d'8 e'8")

    assert format(tuplet) == stringtools.normalize(
        r'''
        \tweak text #tuplet-number::calc-fraction-text
        \times 4/3 {
            c'8
            d'8
            e'8
        }
        '''
        )

    tuplet.toggle_prolation()

    assert format(tuplet) == stringtools.normalize(
        r'''
        \times 2/3 {
            c'4
            d'4
            e'4
        }
        '''
        )

    assert inspect_(tuplet).is_well_formed()


def test_scoretools_Tuplet_toggle_prolation_02():
    '''Change diminution to augmentation.
    '''

    tuplet = Tuplet((2, 3), "c'8 d'8 e'8")

    assert format(tuplet) == stringtools.normalize(
        r'''
        \times 2/3 {
            c'8
            d'8
            e'8
        }
        '''
        )

    tuplet.toggle_prolation()

    assert format(tuplet) == stringtools.normalize(
        r'''
        \tweak text #tuplet-number::calc-fraction-text
        \times 4/3 {
            c'16
            d'16
            e'16
        }
        '''
        )

    assert inspect_(tuplet).is_well_formed()



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_scoretools_Leaf_duration_assign_01():
    r'''Written duration can be assigned a duration.
    '''

    note = Note(1, (1, 4))
    note.written_duration = Duration(1, 8)
    assert note.written_duration == Duration(1, 8)


def test_scoretools_Leaf_duration_assign_02():
    r'''Written duration can be assigned an integer.
    '''

    note = Note(1, (1, 4))
    note.written_duration = 2
    assert note.written_duration == Duration(2, 1)


def test_scoretools_Leaf_duration_assign_03():
    r'''Written duration can be assigned an tuple.
    '''

    note = Note(1, (1, 4))
    note.written_duration = (1, 2)
    assert note.written_duration == Duration(1, 2)



# -*- coding: utf-8 -*-
from abjad import *
import pytest


def test_pitchtools_PitchClassSet___slots___01():
    r'''Named pitch-class set can not be changed after initialization.
    '''

    named_pitch_classes = ['gs', 'a', 'as', 'c', 'cs']
    named_pitch_class_set = pitchtools.PitchClassSet(named_pitch_classes)

    assert pytest.raises(AttributeError,
        "named_pitch_class_set.foo = 'bar'")



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_PitchClassSegment__is_equivalent_under_transposition_01():

    pitch_class_segment_1 = pitchtools.PitchClassSegment(['c', 'e', 'b'])
    pitch_class_segment_2 = pitchtools.PitchClassSegment(['f', 'a', 'e'])
    pitch_class_segment_3 = pitchtools.PitchClassSegment(['f', 'a'])

    assert pitch_class_segment_1._is_equivalent_under_transposition(
        pitch_class_segment_1)
    assert pitch_class_segment_1._is_equivalent_under_transposition(
        pitch_class_segment_2)
    assert not pitch_class_segment_1._is_equivalent_under_transposition(
        pitch_class_segment_3)

    assert pitch_class_segment_2._is_equivalent_under_transposition(
        pitch_class_segment_1)
    assert pitch_class_segment_2._is_equivalent_under_transposition(
        pitch_class_segment_2)
    assert not pitch_class_segment_2._is_equivalent_under_transposition(
        pitch_class_segment_3)

    assert not pitch_class_segment_3._is_equivalent_under_transposition(
        pitch_class_segment_1)
    assert not pitch_class_segment_3._is_equivalent_under_transposition(
        pitch_class_segment_2)
    assert pitch_class_segment_3._is_equivalent_under_transposition(
        pitch_class_segment_3)



# -*- coding: utf-8 -*-
import os
from abjad import *
configuration = systemtools.AbjadConfiguration()
ly_path = os.path.join(
    configuration.abjad_directory,
    'test.ly',
    )
pdf_path = os.path.join(
    configuration.abjad_directory,
    'test.pdf',
    )
paths = [ly_path, pdf_path]


def test_agenttools_PersistenceAgent_as_pdf_01():
    r'''Agent persists PDF file when no PDF file exists.
    '''

    note = Note("c'4")
    with systemtools.FilesystemState(remove=paths):
        result = persist(note).as_pdf(pdf_path)
        assert os.path.isfile(pdf_path)
        assert isinstance(result, tuple)


def test_agenttools_PersistenceAgent_as_pdf_02():
    r'''Agent persists PDF file when equivalent PDF file already exists.
    '''

    note = Note("c'4")
    with systemtools.FilesystemState(remove=paths):
        result = persist(note).as_pdf(pdf_path)
        assert os.path.isfile(pdf_path)
        assert isinstance(result, tuple)
        os.remove(ly_path)
        persist(note).as_pdf(pdf_path)
        assert os.path.isfile(pdf_path)



# -*- coding: utf-8 -*-
from abjad import *


def test_selectiontools_Parentage_root_01():

    tuplet = scoretools.FixedDurationTuplet(Duration(2, 8), "c'8 d'8 e'8")
    staff = Staff([tuplet])
    leaves = select(staff).by_leaf()

    assert inspect_(staff).get_parentage().root is staff
    assert inspect_(tuplet).get_parentage().root is staff
    assert inspect_(leaves[0]).get_parentage().root is staff
    assert inspect_(leaves[1]).get_parentage().root is staff
    assert inspect_(leaves[2]).get_parentage().root is staff



# -*- coding: utf-8 -*-


def all_are_pairs(argument):
    r'''Is true when `argument` is an iterable collection of pairs.
    Otherwise false.

    ..  container:: example

        ::

            >>> mathtools.all_are_pairs([(1, 2), (3, 4), (5, 6), (7, 8)])
            True

        ::

            >>> mathtools.all_are_pairs('foo')
            False

    ..  container:: example

        Is true when `argument` is empty:

        ::

            >>> mathtools.all_are_pairs([])
            True

    Returns true or false.
    '''
    try:
        return all(len(_) == 2 for _ in argument)
    except TypeError:
        return False



# -*- coding: utf-8 -*-
import copy
from abjad.tools.datastructuretools.TypedList import TypedList


class TempoList(TypedList):
    r'''Tempo list.

    ..  container:: example

        ::

            >>> tempos = indicatortools.TempoList([
            ...     (Duration(1, 8), 72, 'Andante'),
            ...     (Duration(1, 8), 84, 'Allegro'),
            ...     ])

        ::

            >>> for tempo in tempos:
            ...     tempo
            ...
            Tempo(reference_duration=Duration(1, 8), units_per_minute=72, textual_indication='Andante')
            Tempo(reference_duration=Duration(1, 8), units_per_minute=84, textual_indication='Allegro')

    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Collections'

    __slots__ = (
        )

    ### SPECIAL METHODS ###

    def __contains__(self, argument):
        r'''Is true when tempo list contains `argument`.

        ..  container:: example

            ::

                >>> tempos = indicatortools.TempoList([
                ...     (Duration(1, 8), 72),
                ...     ((1, 8), 84, 'Allegro'),
                ...     ])

            ::

                >>> Tempo((1, 8), 72) in tempos
                True

            ::

                >>> ((1, 8), 72) in tempos
                True

            ::

                >>> (Duration(1, 8), 72) in tempos
                True

            ::

                >>> ((1, 8), 84, 'Allegro') in tempos
                True

            ::

                >>> ((1, 8), 96) in tempos
                False

        Returns true or false.
        '''
        superclass = super(TempoList, self)
        return superclass.__contains__(argument)

    def __format__(self, format_specification=''):
        r'''Formats tempo list.

        ..  container:: example

            ::

                >>> tempos = indicatortools.TempoList([
                ...     (Duration(1, 8), 72),
                ...     ((1, 8), 84, 'Allegro'),
                ...     ])

            ::

                >>> f(tempos)
                indicatortools.TempoList(
                    [
                        indicatortools.Tempo(
                            reference_duration=durationtools.Duration(1, 8),
                            units_per_minute=72,
                            ),
                        indicatortools.Tempo(
                            reference_duration=durationtools.Duration(1, 8),
                            units_per_minute=84,
                            textual_indication='Allegro',
                            ),
                        ]
                    )

        Returns string.
        '''
        superclass = super(TempoList, self)
        return superclass.__format__(format_specification=format_specification)

    def __illustrate__(self):
        r'''Illustrates tempos.

        ..  container:: example

            ::

                >>> tempos = indicatortools.TempoList([
                ...     (Duration(1, 8), 72),
                ...     ((1, 8), 84, 'Allegro'),
                ...     ])

            ::

                >>> show(tempos) # doctest: +SKIP

            ..  doctest::

                >>> lilypond_file = tempos.__illustrate__()
                >>> f(lilypond_file[Score])
                \new Score \with {
                    \override BarLine.transparent = ##t
                    \override BarNumber.stencil = ##f
                    \override Clef.stencil = ##f
                    \override NoteHead.no-ledgers = ##t
                    \override NoteHead.transparent = ##t
                    \override StaffSymbol.transparent = ##t
                    \override Stem.transparent = ##t
                    \override TimeSignature.stencil = ##f
                } <<
                    \new Staff {
                        \time 2/4
                        \break
                        c'2
                        \tempo 8=72
                        \break
                        c'2
                        \tempo Allegro 8=84
                        \break
                        c'2
                    }
                >>

        Returns LilyPond file.
        '''
        import abjad
        staff = abjad.Staff()
        score = abjad.Score([staff])
        time_signature = abjad.TimeSignature((2, 4))
        abjad.attach(time_signature, staff)
        # the zero note avoids a lilypond spacing problem:
        # score-initial tempo indications slip to the left
        zero_note = abjad.Note("c'2")
        staff.append(zero_note)
        command = abjad.LilyPondCommand('break')
        abjad.attach(command, zero_note)
        for tempo in self.items:
            note = abjad.Note("c'2")
            abjad.attach(tempo, note)
            staff.append(note)
            command = abjad.LilyPondCommand('break')
            abjad.attach(command, note)
        abjad.override(score).bar_line.transparent = True
        abjad.override(score).bar_number.stencil = False
        abjad.override(score).clef.stencil = False
        abjad.override(score).note_head.no_ledgers = True
        abjad.override(score).note_head.transparent = True
        abjad.override(score).staff_symbol.transparent = True
        abjad.override(score).stem.transparent = True
        abjad.override(score).time_signature.stencil = False
        lilypond_file = abjad.LilyPondFile.new(score)
        lilypond_file.layout_block.indent = 0
        lilypond_file.layout_block.ragged_right = True
        lilypond_file.items.remove(lilypond_file['paper'])
        lilypond_file.header_block.tagline = False
        return lilypond_file

    ### PRIVATE PROPERTIES ###

    @property
    def _item_coercer(self):
        def coerce_(argument):
            if argument is None:
                tempo = indicatortools.Tempo()
            elif isinstance(argument, tuple):
                tempo = indicatortools.Tempo(*argument)
            elif isinstance(argument, indicatortools.Tempo):
                tempo = copy.copy(argument)
            else:
                raise TypeError(repr(argument))
            return tempo
        from abjad.tools import indicatortools
        return coerce_

    ### PUBLIC ###

    def append(self, tempo):
        r'''Appends tempo.

        ..  container:: example

            ::

                >>> tempos_1 = indicatortools.TempoList([((1, 8), 72)])
                >>> tempos_1.append(((1, 8), 84))
                >>> tempos_2 = indicatortools.TempoList([
                ...    ((1, 8), 72),
                ...    ((1, 8), 84),
                ...    ])

            ::

                >>> tempos_1 == tempos_2
                True

        Returns none.
        '''
        superclass = super(TempoList, self)
        return superclass.append(tempo)



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_spannertools_Hairpin_01():
    r'''Hairpins span adjacent leaves.
    '''

    staff = Staff([Note(n, (1, 8)) for n in range(8)])
    crescendo = Crescendo()
    attach(crescendo, staff[:4])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8 \<
            cs'8
            d'8
            ef'8 \!
            e'8
            f'8
            fs'8
            g'8
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


def test_spannertools_Hairpin_02():
    r'''Hairpins and dynamics apply separately.
    '''

    staff = Staff([Note(n, (1, 8)) for n in range(8)])
    crescendo = Crescendo()
    attach(crescendo, staff[:4])
    dynamic = Dynamic('p')
    attach(dynamic, staff[0])
    dynamic = Dynamic('f')
    attach(dynamic, staff[3])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8 \p \<
            cs'8
            d'8
            ef'8 \f
            e'8
            f'8
            fs'8
            g'8
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


def test_spannertools_Hairpin_03():

    staff = Staff([Note(n, (1, 8)) for n in range(8)])
    crescendo = Crescendo()
    attach(crescendo, staff[:4])
    dynamic = Dynamic('p')
    attach(dynamic, staff[2])

    assert not inspect_(staff).is_well_formed()


def test_spannertools_Hairpin_04():
    r'''Apply back-to-back hairpins separately.
    '''

    staff = Staff([Note(n, (1, 8)) for n in range(8)])
    dynamic = Dynamic('p')
    attach(dynamic, staff[0])
    crescendo = Crescendo()
    attach(crescendo, staff[0:3])
    dynamic = Dynamic('f')
    attach(dynamic, staff[2])
    decrescendo = Decrescendo()
    attach(decrescendo, staff[2:5])
    dynamic = Dynamic('p')
    attach(dynamic, staff[4])
    crescendo = Crescendo()
    attach(crescendo, staff[4:7])
    dynamic = Dynamic('f')
    attach(dynamic, staff[6])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8 \p \<
            cs'8
            d'8 \f \>
            ef'8
            e'8 \p \<
            f'8
            fs'8 \f
            g'8
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


def test_spannertools_Hairpin_05():
    r'''Hairpins format rests.
    '''

    staff = Staff(Rest((1, 8)) * 4 + [Note(n, (1, 8)) for n in range(4, 8)])
    crescendo = Crescendo(include_rests=True)
    attach(crescendo, staff[:])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            r8 \<
            r8
            r8
            r8
            e'8
            f'8
            fs'8
            g'8 \!
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


def test_spannertools_Hairpin_06():
    r'''Trimmed hairpins with dynamics behave as expected.
    '''

    staff = Staff([Note(n, (1, 8)) for n in range(8)])
    rest = Rest(staff[0])
    staff[0] = rest
    rest = Rest(staff[-1])
    staff[-1] = rest
    hairpin = Hairpin(
        descriptor='p < f',
        include_rests=False,
        )
    attach(hairpin, staff[:])

    prototype = Hairpin
    spanner = inspect_(staff[0]).get_spanner(prototype=prototype)
    assert len(spanner.components) == len(staff)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            r8
            cs'8 \< \p
            d'8
            ef'8
            e'8
            f'8
            fs'8 \f
            r8
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


def test_spannertools_Hairpin_07():
    
    staff = Staff("c'4 d'4 e'4 f'4")
    attach(Dynamic('p'), staff[0])
    attach(Hairpin('<'), staff[:])
    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'4 \p \<
            d'4
            e'4
            f'4 \!
        }
        '''
        )


# -*- coding: utf-8 -*-
from abjad.tools.abctools import AbjadValueObject


class LengthSelectorCallback(AbjadValueObject):
    r'''Length selector callback.
    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Callbacks'

    __slots__ = (
        '_length',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        length=1,
        ):
        from abjad.tools import selectortools
        prototype = (
            int,
            selectortools.LengthInequality,
            )
        assert isinstance(length, prototype)
        self._length = length

    ### SPECIAL METHODS ###

    def __call__(self, argument, rotation=None):
        r'''Iterates tuple `argument`.

        Returns tuple in which each item is a selection or component.
        '''
        from abjad.tools import selectortools
        assert isinstance(argument, tuple), repr(argument)
        inequality = self.length
        if not isinstance(inequality, selectortools.LengthInequality):
            inequality = selectortools.LengthInequality(
                length=inequality,
                operator_string='==',
                )
        result = []
        for subexpr in argument:
            if inequality(subexpr):
                result.append(subexpr)
        return tuple(result)

    ### PUBLIC PROPERTIES ###

    @property
    def length(self):
        r'''Gets length selector callback length.

        Returns length.
        '''
        return self._length



# -*- coding: utf-8 -*-
from abjad import *
from abjad.tools.abctools.AbjadObject import AbjadObject


def test_custom_score_template_class_01():
    r'''Score template with named contexts.
    '''

    class NamedContextScoreTemplate(AbjadObject):

        ### INITIALIZER ###

        def __init__(self):
            pass

        ### SPECIAL METHODS ###

        def __call__(self):
            voice = scoretools.Voice(name='Blue Voice')
            staff = scoretools.Staff(name='Red Staff')
            score = scoretools.Score(name='Green Score')
            staff.append(voice)
            score.append(staff)
            return score

    named_context_score_template = NamedContextScoreTemplate()
    score = named_context_score_template()

    assert format(score) == stringtools.normalize(
        r'''
        \context Score = "Green Score" <<
            \context Staff = "Red Staff" {
                \context Voice = "Blue Voice" {
                }
            }
        >>
        '''
        )


def test_custom_score_template_class_02():
    r'''Score template with custom (voice and staff) contexts.

    CAUTION: always use built-in LilyPond score context; do not rename.
    '''

    class CustomContextScoreTemplate(AbjadObject):

        ### INITIALIZER ###

        def __init__(self):
            pass

        ### SPECIAL METHODS ###

        def __call__(self):
            custom_voice = scoretools.Voice(context_name='CustomVoice')
            custom_staff = scoretools.Staff(context_name='CustomStaff')
            score = scoretools.Score()
            custom_staff.append(custom_voice)
            score.append(custom_staff)
            return score

    custom_context_score_template = CustomContextScoreTemplate()
    score = custom_context_score_template()

    assert format(score) == stringtools.normalize(
        r'''
        \new Score <<
            \new CustomStaff {
                \new CustomVoice {
                }
            }
        >>
        '''
        )

    # here's how to properly override with externalized layout

    score = custom_context_score_template()
    score[0][0].append("c'4 ( d'4 e'4 f'4 )")
    lilypond_file = lilypondfiletools.LilyPondFile.new(score)

    context_block = lilypondfiletools.ContextBlock(
        source_context_name='Voice',
        type_='Engraver_group',
        name='CustomVoice',
        alias='Voice',
        )
    lilypond_file.layout_block.items.append(context_block)
    override(context_block).note_head.color = 'green'
    override(context_block).stem.color = 'green'

    context_block = lilypondfiletools.ContextBlock(
        source_context_name='Staff',
        type_='Engraver_group',
        name='CustomStaff',
        alias='Staff',
        )
    lilypond_file.layout_block.items.append(context_block)
    context_block.accepts_commands.append('CustomVoice')
    override(context_block).staff_symbol.color = 'red'

    context_block = lilypondfiletools.ContextBlock(
        source_context_name='Score',
        )
    lilypond_file.layout_block.items.append(context_block)
    context_block.accepts_commands.append('CustomStaff')

    assert format(lilypond_file.layout_block) == stringtools.normalize(
        r'''
        \layout {
            \context {
                \Voice
                \name CustomVoice
                \type Engraver_group
                \alias Voice
                \override NoteHead.color = #green
                \override Stem.color = #green
            }
            \context {
                \Staff
                \name CustomStaff
                \type Engraver_group
                \alias Staff
                \accepts CustomVoice
                \override StaffSymbol.color = #red
            }
            \context {
                \Score
                \accepts CustomStaff
            }
        }
        '''
        )

    assert format(lilypond_file.score_block) == stringtools.normalize(
        r'''
        \score {
            \new Score <<
                \new CustomStaff {
                    \new CustomVoice {
                        c'4 (
                        d'4
                        e'4
                        f'4 )
                    }
                }
            >>
        }
        '''
        )



# -*- coding: utf-8 -*-
import inspect
import pytest
from abjad.tools import documentationtools
from abjad.tools import systemtools


ignored_classes = (
    systemtools.StorageFormatAgent,
    systemtools.FormatSpecification,
    )

classes = documentationtools.list_all_abjad_classes(
    ignored_classes=ignored_classes,
    )


@pytest.mark.parametrize('class_', classes)
def test_abjad___rmul___01(class_):
    r'''All classes implementing __mul__ also implement __rmul__.
    '''
    if inspect.isabstract(class_):
        return
    if hasattr(class_, '__mul__'):
        assert hasattr(class_, '__rmul__')



# -*- coding: utf-8 -*-
import sys
import sphinx_rtd_theme
from sphinx.highlighting import PygmentsBridge
from pygments.formatters.latex import LatexFormatter
from abjad import abjad_configuration


class CustomLatexFormatter(LatexFormatter):
    def __init__(self, **options):
        super(CustomLatexFormatter, self).__init__(**options)
        self.verboptions = r'''formatcom=\footnotesize'''

PygmentsBridge.latex_formatter = CustomLatexFormatter

### CORE ###
add_function_parentheses = True
copyright = u'2008-2017, Trevor Bača & Josiah Wolf Oberholtzer'
exclude_patterns = []
extensions = [
    'sphinx.ext.autodoc',
    'sphinx.ext.autosummary',
    'sphinx.ext.coverage',
    'sphinx.ext.graphviz',
    'sphinx.ext.intersphinx',
    'sphinx.ext.todo',
    'abjad.docs.ext.abjadbook',
    ]
master_doc = 'index'
project = u'Abjad'
pygments_style = 'sphinx'
release = abjad_configuration.get_abjad_version_string()
source_suffix = '.rst'
templates_path = ['_templates']
version = abjad_configuration.get_abjad_version_string()
### HTML ###
html_domain_indices = False
html_favicon = '_static/favicon.ico'
html_last_updated_fmt = '%b %d, %Y'
html_logo = '_static/abjad-logo.png'
html_show_sourcelink = True
html_static_path = ['_static']
html_theme = "sphinx_rtd_theme"
html_theme_path = [sphinx_rtd_theme.get_html_theme_path()]
html_use_index = False
html_use_smartypants = True
### HTML HELP ###
htmlhelp_basename = 'Abjaddoc'
### LATEX ###
latex_elements = {
    'inputenc': r'\usepackage[utf8x]{inputenc}',
    'utf8extra': '',
    'papersize': 'a4paper',
    'pointsize': '10pt',
    'preamble': r'''
    \usepackage{upquote}
    \pdfminorversion=5
    \setcounter{tocdepth}{2}
    \definecolor{VerbatimColor}{rgb}{0.95,0.95,0.95}
    \definecolor{VerbatimBorderColor}{rgb}{1.0,1.0,1.0}
    \hypersetup{unicode=true}
    ''',
    }
latex_documents = [
    (
        'index',
        'Abjad.tex',
        u'Abjad Documentation',
        u'Trevor Bača & Josiah Wolf Oberholtzer',
        'manual',
        ),
    (
        'api/index',
        'AbjadAPI.tex',
        u'Abjad API',
        u'Trevor Bača & Josiah Wolf Oberholtzer',
        'manual',
        ),
    ]
#latex_use_parts = True
latex_toplevel_sectioning = 'chapter' # just guessing?
### MAN ###
man_pages = [
    (
        'index',
        'abjad',
        u'Abjad Documentation',
        [u'2008-2017, Trevor Bača & Josiah Wolf Oberholtzer'],
        1,
        )
    ]
### TEXINFO ###
texinfo_documents = [
    (
        'index',
        'Abjad',
        u'Abjad Documentation',
        u'2008-2017, Trevor Bača & Josiah Wolf Oberholtzer',
        'Abjad',
        'One line description of project.',
        'Miscellaneous',
        ),
    ]
### EXTENSIONS ###
abjadbook_ignored_documents = ()
autodoc_member_order = 'groupwise'
graphviz_dot_args = ['-s32']
graphviz_output_format = 'svg'
intersphinx_mapping = {
    'python': (
        'http://docs.python.org/{}.{}'.format(*sys.version_info[:2]), None),
    }
todo_include_todos = True



# -*- coding: utf-8 -*-
from abjad import *


def test_rhythmtreetools_RhythmTreeContainer___contains___01():

    leaf_a = rhythmtreetools.RhythmTreeLeaf(preprolated_duration=1)
    leaf_b = rhythmtreetools.RhythmTreeLeaf(preprolated_duration=1)
    leaf_c = rhythmtreetools.RhythmTreeLeaf(preprolated_duration=1)

    subcontainer = rhythmtreetools.RhythmTreeContainer(preprolated_duration=1, children=[leaf_b])

    container = rhythmtreetools.RhythmTreeContainer(preprolated_duration=1, children=[leaf_a, subcontainer])

    assert leaf_a in container
    assert leaf_b not in container
    assert leaf_c not in container



# -*- coding: utf-8 -*-
from abjad.tools import stringtools
from abjad.tools.spannertools.Spanner import Spanner
from abjad.tools.topleveltools import override
from abjad.tools.topleveltools import iterate


class Slur(Spanner):
    r'''Slur.

    ..  container:: example

        Slurs four notes:

        ::

            >>> staff = Staff("c'4 d' e' f'")
            >>> attach(Slur(), staff[:])
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                c'4 (
                d'4
                e'4
                f'4 )
            }

    ..  container:: example

        Requires at least two leaves:

        ::

            >>> staff = Staff("c'4 d' e' f'")
            >>> attach(Slur(), staff[:1])
            Traceback (most recent call last):
                ...
            Exception: Slur() attachment test fails for Selection([Note("c'4")]).

    Formats LilyPond ``(`` command on first leaf in spanner.

    Formats LilyPond ``)`` command on last leaf in spanner.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_direction',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        direction=None,
        overrides=None,
        ):
        Spanner.__init__(
            self,
            overrides=overrides,
            )
        direction = stringtools.to_tridirectional_lilypond_symbol(
            direction)
        self._direction = direction

    ### PRIVATE METHODS ###

    def _attachment_test_all(self, component_expression):
        return self._at_least_two_leaves(component_expression)

    def _copy_keyword_args(self, new):
        new._direction = self.direction

    def _get_lilypond_format_bundle(self, leaf):
        lilypond_format_bundle = self._get_basic_lilypond_format_bundle(leaf)
        if self._is_my_only_leaf(leaf):
            pass
        elif self._is_my_first_leaf(leaf):
            if self.direction is not None:
                string = '{} ('.format(self.direction)
            else:
                string = '('
            lilypond_format_bundle.right.spanner_starts.append(string)
        elif self._is_my_last_leaf(leaf):
            string = ')'
            lilypond_format_bundle.right.spanner_stops.append(string)
        return lilypond_format_bundle

    ### PUBLIC PROPERTIES ###

    @property
    def direction(self):
        r'''Gets direction.

        ..  container:: example

            Forces slur above staff:

            ::

                >>> staff = Staff("c'8 d'8 e'8 f'8")
                >>> slur = spannertools.Slur(direction=Up)
                >>> attach(slur, staff[:])
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff {
                    c'8 ^ (
                    d'8
                    e'8
                    f'8 )
                    }

        ..  container:: example

            Forces slur below staff:

            ::

                >>> staff = Staff("c'8 d'8 e'8 f'8")
                >>> slur = spannertools.Slur(direction=Down)
                >>> attach(slur, staff[:])
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff {
                    c'8 _ (
                    d'8
                    e'8
                    f'8 )
                }

        ..  container:: example

            Positions slur according to LilyPond defaults:

            ::

                >>> staff = Staff("c'8 d'8 e'8 f'8")
                >>> slur = spannertools.Slur(direction=None)
                >>> attach(slur, staff[:])
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff {
                    c'8 (
                    d'8
                    e'8
                    f'8 )
                }

        Defaults to none.

        Set to up, down or none.

        Returns up, down or none.
        '''
        return self._direction



# -*- coding: utf-8 -*-
#! /usr/bin/env python

if __name__ == '__main__':
    import sys
    from abjad import Duration
    from abjad import show
    from abjad.demos.ferneyhough import make_lilypond_file

    arguments = sys.argv[1:]
    if len(arguments) != 3:
        print('USAGE: [tuplet_duration, row_count, column_count]:')
        print('\ti.e.: /main.py 1/4 11 6')
        sys.exit(0)

    tuplet_duration = Duration(arguments[0])
    row_count = int(arguments[1])
    column_count = int(arguments[2])

    assert 0 < tuplet_duration
    assert 0 < row_count
    assert 0 < column_count

    lilypond_file = make_lilypond_file(tuplet_duration, row_count, column_count)
    show(lilypond_file)



# -*- coding: utf-8 -*-
from abjad.tools.topleveltools import override
from abjad.tools.topleveltools import set_


def make_text_alignment_example_lilypond_file(music=None):
    r'''Makes text-alignment example LilyPond file.

        >>> score = Score([Staff('c d e f')])
        >>> lilypond_file = documentationtools.make_text_alignment_example_lilypond_file(score)

    ..  doctest::

        >>> print(format(lilypond_file)) # doctest: +SKIP
        % Abjad revision 5651
        % 2012-05-19 10:04

        \version "2.15.37"
        \language "english"

        #(set-global-staff-size 18)

        \layout {
            indent = #0
            ragged-right = ##t
            \context {
                \Score
                \remove Bar_number_engraver
                \remove Default_bar_line_engraver
                \override Clef.transparent = ##t
                \override SpacingSpanner.strict-grace-spacing = ##t
                \override SpacingSpanner.strict-note-spacing = ##t
                \override SpacingSpanner.uniform-stretching = ##t
                \override TextScript.staff-padding = #4
                proportionalNotationDuration = #(ly:make-moment 1 32)
            }
        }

        \paper {
            bottom-margin = #10
            left-margin = #10
            line-width = #150
            system-system-spacing = #'(
                (basic-distance . 0) (minimum-distance . 0) (padding . 15) (stretchability . 0))
        }

        \score {
            \new Score <<
                \new Staff {
                    c4
                    d4
                    e4
                    f4
                }
            >>
        }

    Returns LilyPond file.
    '''
    from abjad.tools import lilypondfiletools
    from abjad.tools import schemetools

    lilypond_file = lilypondfiletools.LilyPondFile.new(
        music=music,
        global_staff_size=18,
        )

    lilypond_file.layout_block.indent = 0
    lilypond_file.layout_block.ragged_right = True

    context_block = lilypondfiletools.ContextBlock(
        source_context_name='Score',
        )
    lilypond_file.layout_block.items.append(context_block)

    context_block.remove_commands.append('Bar_number_engraver')
    context_block.remove_commands.append('Default_bar_line_engraver')
    override(context_block).clef.transparent = True
    override(context_block).spacing_spanner.strict_grace_spacing = True
    override(context_block).spacing_spanner.strict_note_spacing = True
    override(context_block).spacing_spanner.uniform_stretching = True
    override(context_block).text_script.staff_padding = 4
    override(context_block).time_signature.transparent = True
    moment = schemetools.SchemeMoment((1, 32))
    set_(context_block).proportionalNotationDuration = moment

    lilypond_file.paper_block.bottom_margin = 10
    lilypond_file.paper_block.left_margin = 10
    lilypond_file.paper_block.line_width = 150

    vector = schemetools.make_spacing_vector(0, 0, 15, 0)
    lilypond_file.paper_block.system_system_spacing = vector

    return lilypond_file



# -*- coding: utf-8 -*-
from abjad.tools import systemtools
from abjad.tools.abctools import AbjadValueObject


class PostscriptOperator(AbjadValueObject):
    r'''Postscript operator.

    ..  container:: example

        ::

            >>> operator = markuptools.PostscriptOperator('rmoveto', 1, 1.5)
            >>> print(format(operator))
            markuptools.PostscriptOperator('rmoveto', 1, 1.5)

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_name',
        '_arguments',
        )

    ### INITIALIZER ###

    def __init__(self, name='stroke', *arguments):
        name = str(name)
        self._name = name
        if arguments:
            self._arguments = tuple(arguments)
        else:
            self._arguments = None

    ### SPECIAL METHODS ###

    def __str__(self):
        r'''Gets string representation of Postscript operator.

        ::

            >>> operator = markuptools.PostscriptOperator('rmoveto', 1, 1.5)
            >>> str(operator)
            '1 1.5 rmoveto'

        Returns string.
        '''
        from abjad.tools import markuptools
        parts = []
        if self.arguments:
            for argument in self.arguments:
                parts.append(markuptools.Postscript._format_argument(argument))
        parts.append(self.name)
        string = ' '.join(parts)
        return string

    ### PRIVATE METHODS ###

    def _get_format_specification(self):
        values = [self.name] + list(self.arguments or ())
        return systemtools.FormatSpecification(
            client=self,
            storage_format_args_values=values,
            storage_format_is_indented=False,
            storage_format_kwargs_names=[],
            )

    ### PUBLIC PROPERTIES ###

    @property
    def arguments(self):
        r'''Gets Postscript operator arguments.

        Returns tuple or none.
        '''
        return self._arguments

    @property
    def name(self):
        r'''Gets Postscript operator name.

        Returns string.
        '''
        return self._name



# -*- coding: utf-8 -*-
import copy
import numbers


def transpose_pitch_carrier_by_interval(pitch_carrier, interval):
    '''Transposes `pitch_carrier` by named `interval`.

    ::

        >>> chord = Chord("<c' e' g'>4")

    ::

        >>> pitchtools.transpose_pitch_carrier_by_interval(
        ...     chord, '+m2')
        Chord("<df' f' af'>4")

    Transpose `pitch_carrier` by numbered `interval`:

    ::

        >>> chord = Chord("<c' e' g'>4")

    ::

        >>> pitchtools.transpose_pitch_carrier_by_interval(chord, 1)
        Chord("<cs' f' af'>4")

    Returns non-pitch-carrying input unchaged:

    ::

        >>> rest = Rest('r4')

    ::

        >>> pitchtools.transpose_pitch_carrier_by_interval(rest, 1)
        Rest('r4')

    Return `pitch_carrier`.
    '''
    from abjad.tools import pitchtools
    from abjad.tools import scoretools

    def _transpose_pitch_by_named_interval(pitch, mdi):
        pitch_number = pitch.pitch_number + mdi.semitones
        diatonic_pitch_class_number = \
            (pitch.diatonic_pitch_class_number + mdi.staff_spaces) % 7
        diatonic_pitch_class_name = \
            pitchtools.PitchClass._diatonic_pitch_class_number_to_diatonic_pitch_class_name[
                diatonic_pitch_class_number]
        named_pitch = pitchtools.NamedPitch(
            pitch_number, diatonic_pitch_class_name)
        return type(pitch)(named_pitch)

    def _transpose_pitch_carrier_by_named_interval(
        pitch_carrier, named_interval):
        mdi = pitchtools.NamedInterval(named_interval)
        if isinstance(pitch_carrier, pitchtools.Pitch):
            return _transpose_pitch_by_named_interval(
                pitch_carrier, mdi)
        elif isinstance(pitch_carrier, scoretools.Note):
            new_note = copy.copy(pitch_carrier)
            new_pitch = _transpose_pitch_by_named_interval(
                pitch_carrier.written_pitch, mdi)
            new_note.written_pitch = new_pitch
            return new_note
        elif isinstance(pitch_carrier, scoretools.Chord):
            new_chord = copy.copy(pitch_carrier)
            for new_nh, old_nh in \
                zip(new_chord.note_heads, pitch_carrier.note_heads):
                new_pitch = _transpose_pitch_by_named_interval(
                    old_nh.written_pitch, mdi)
                new_nh.written_pitch = new_pitch
            return new_chord
        else:
            return pitch_carrier

    def _transpose_pitch_carrier_by_numbered_interval(
        pitch_carrier, numbered_interval):
        mci = pitchtools.NumberedInterval(numbered_interval)
        if isinstance(pitch_carrier, pitchtools.Pitch):
            number = pitch_carrier.pitch_number + mci.semitones
            return type(pitch_carrier)(number)
        elif isinstance(pitch_carrier, numbers.Number):
            pitch_carrier = pitchtools.NumberedPitch(pitch_carrier)
            result = _transpose_pitch_carrier_by_numbered_interval(
                pitch_carrier, mci)
            return result.pitch_number
        elif isinstance(pitch_carrier, scoretools.Note):
            new_note = copy.copy(pitch_carrier)
            number = pitchtools.NumberedPitch(
                pitch_carrier.written_pitch).pitch_number
            number += mci.number
            new_pitch = pitchtools.NamedPitch(number)
            new_note.written_pitch = new_pitch
            return new_note
        elif isinstance(pitch_carrier, scoretools.Chord):
            new_chord = copy.copy(pitch_carrier)
            pairs = zip(new_chord.note_heads, pitch_carrier.note_heads)
            for new_nh, old_nh in pairs:
                number = \
                    pitchtools.NumberedPitch(old_nh.written_pitch).pitch_number
                number += mci.number
                new_pitch = pitchtools.NamedPitch(number)
                new_nh.written_pitch = new_pitch
            return new_chord
        else:
            return pitch_carrier

    diatonic_types = (pitchtools.NamedInterval, str)
    if isinstance(interval, diatonic_types):
        interval = \
            pitchtools.NamedInterval(interval)
        return _transpose_pitch_carrier_by_named_interval(
            pitch_carrier, interval)
    else:
        interval = \
            pitchtools.NumberedInterval(interval)
        return _transpose_pitch_carrier_by_numbered_interval(
            pitch_carrier, interval)



# -*- coding: utf-8 -*-
from abjad import *
from abjad.tools.pitchtools import Registration
from abjad.tools.pitchtools import RegistrationList


def test_pitchtools_RegistrationList___repr___01():

    registrations_1 = pitchtools.RegistrationList([[('[A0, C8]', -18)]])
    registrations_2 = eval(repr(registrations_1))

    assert isinstance(registrations_1, pitchtools.RegistrationList)
    assert isinstance(registrations_2, pitchtools.RegistrationList)
    assert registrations_1 == registrations_2



# -*- coding: utf-8 -*-
import copy
from abjad import *

def test_quantizationtools_UnweightedSearchTree__generate_all_subdivision_commands_01():

    definition = {
            2: {
                2: {
                    2: None
                },
                3: None
            },
            5: None
        }
    search_tree = quantizationtools.UnweightedSearchTree(definition)

    q_grid = quantizationtools.QGrid()
    a = quantizationtools.QEventProxy(quantizationtools.SilentQEvent(0,      ['A'], index=1), 0, 1)
    b = quantizationtools.QEventProxy(quantizationtools.SilentQEvent((1, 5), ['B'], index=2), 0, 1)
    c = quantizationtools.QEventProxy(quantizationtools.SilentQEvent((1, 4), ['C'], index=3), 0, 1)
    d = quantizationtools.QEventProxy(quantizationtools.SilentQEvent((1, 3), ['D'], index=4), 0, 1)
    e = quantizationtools.QEventProxy(quantizationtools.SilentQEvent((2, 5), ['E'], index=5), 0, 1)
    f = quantizationtools.QEventProxy(quantizationtools.SilentQEvent((1, 2), ['F'], index=6), 0, 1)
    g = quantizationtools.QEventProxy(quantizationtools.SilentQEvent((3, 5), ['G'], index=7), 0, 1)
    h = quantizationtools.QEventProxy(quantizationtools.SilentQEvent((2, 3), ['H'], index=8), 0, 1)
    i = quantizationtools.QEventProxy(quantizationtools.SilentQEvent((3, 4), ['I'], index=9), 0, 1)
    j = quantizationtools.QEventProxy(quantizationtools.SilentQEvent((4, 5), ['J'], index=10), 0, 1)
    k = quantizationtools.QEventProxy(quantizationtools.SilentQEvent(1,      ['K'], index=11), 0, 1)
    q_grid.fit_q_events([a, b, c, d, e, f, g, h, i, j, k])

    commands = search_tree._generate_all_subdivision_commands(q_grid)
    assert commands == (
        ((0, (1, 1)),),
        ((0, (1, 1, 1, 1, 1)),)
        )

    new_q_grid = copy.copy(q_grid)
    q_event_proxies = new_q_grid.subdivide_leaves(commands[0])
    new_q_grid.fit_q_events(q_event_proxies)
    new_q_grid.sort_q_events_by_index()
    new_commands = search_tree._generate_all_subdivision_commands(new_q_grid)

    assert new_q_grid.leaves[0].q_event_proxies == [a, b, c]
    assert new_q_grid.leaves[1].q_event_proxies == [d, e, f, g, h, i]
    assert new_q_grid.leaves[2].q_event_proxies == [j, k]
    assert new_commands == (
        ((0, (1, 1)), (1, (1, 1))),
        ((0, (1, 1)), (1, (1, 1, 1))),
        ((0, (1, 1, 1)), (1, (1, 1))),
        ((0, (1, 1, 1)), (1, (1, 1, 1)))
        )

    new_q_grid = copy.copy(q_grid)
    q_event_proxies = new_q_grid.subdivide_leaves(commands[1])
    new_q_grid.fit_q_events(q_event_proxies)
    new_q_grid.sort_q_events_by_index()
    new_commands = search_tree._generate_all_subdivision_commands(new_q_grid)

    assert new_q_grid.leaves[0].q_event_proxies == [a]
    assert new_q_grid.leaves[1].q_event_proxies == [b, c]
    assert new_q_grid.leaves[2].q_event_proxies == [d, e, f]
    assert new_q_grid.leaves[3].q_event_proxies == [g, h]
    assert new_q_grid.leaves[4].q_event_proxies == [i, j]
    assert new_q_grid.leaves[5].q_event_proxies == [k]
    assert new_commands == ()



# -*- encoding: utf-8 -*-
from abjad.tools import indicatortools
from abjad.tools import scoretools
from abjad.tools.spannertools.Spanner import Spanner
from abjad.tools.topleveltools import inspect_


class ClefSpanner(Spanner):
    r'''Clef spanner.

    ..  container:: example

        Attaches percussion clef spanner to notes in middle of staff:

        ::

            >>> staff = Staff("c' d' e' f' g' a' b' c''")
            >>> clef = Clef('treble')
            >>> attach(clef, staff[0])
            >>> clef_spanner = spannertools.ClefSpanner('percussion')
            >>> attach(clef_spanner, staff[2:-2])
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                \clef "treble"
                c'4
                d'4
                \clef "percussion"
                e'4
                f'4
                g'4
                a'4
                \clef "treble"
                b'4
                c''4
            }

    ..  container:: example

        Attaches two clef spanners to notes in middle of staff. Only the first
        clef spanner formats a new clef:

        ::

            >>> staff = Staff("r4 c'4 d'4 r4 e'4 f'4 r4")
            >>> clef = Clef('treble')
            >>> attach(clef, staff[0])
            >>> clef_spanner = spannertools.ClefSpanner('percussion')
            >>> attach(clef_spanner, staff[1:3])
            >>> clef_spanner = spannertools.ClefSpanner('percussion')
            >>> attach(clef_spanner, staff[4:6])
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                \clef "treble"
                r4
                \clef "percussion"
                c'4
                d'4
                r4
                e'4
                f'4
                \clef "treble"
                r4
            }

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_clef',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        clef='percussion',
        overrides=None,
        ):
        Spanner.__init__(
            self,
            overrides=overrides,
            )
        clef = indicatortools.Clef(clef)
        self._clef = clef

    ### SPECIAL METHODS ###

    def __getnewargs__(self):
        r'''Gets new arguments of spanner.

        Returns empty tuple.
        '''
        return (
            self.clef,
            )

    ### PRIVATE METHODS ###

    def _copy_keyword_args(self, new):
        new._clef = self.clef

    def _get_lilypond_format_bundle(self, leaf):
        lilypond_format_bundle = self._get_basic_lilypond_format_bundle(leaf)
        prototype = (scoretools.Note, scoretools.Chord, type(None))
        first_leaf = self._get_leaves()[0]
        current_clef = inspect_(first_leaf).get_effective(indicatortools.Clef)
        set_clef = False
        reset_clef = False
        if self._is_my_only_leaf(leaf):
            if self.clef != current_clef:
                set_clef = True
                reset_clef = True

            previous_leaf = inspect_(leaf).get_leaf(-1)
            while not isinstance(previous_leaf, prototype):
                previous_leaf = inspect_(previous_leaf).get_leaf(-1)
            if previous_leaf is not None:
                spanners = inspect_(previous_leaf).get_spanners(type(self))
                spanners = tuple(spanners)
                if spanners:
                    if spanners[0].clef == self.clef:
                        set_clef = False
            next_leaf = inspect_(leaf).get_leaf(1)
            while not isinstance(next_leaf, prototype):
                next_leaf = inspect_(next_leaf).get_leaf(1)
            if next_leaf is not None:
                spanners = inspect_(next_leaf).get_spanners(type(self))
                spanners = tuple(spanners)
                if spanners:
                    if spanners[0].clef == self.clef:
                        reset_clef = False
        elif self._is_my_first_leaf(leaf):
            if self.clef != current_clef:
                set_clef = True
            previous_leaf = inspect_(leaf).get_leaf(-1)
            while not isinstance(previous_leaf, prototype):
                previous_leaf = inspect_(previous_leaf).get_leaf(-1)
            if previous_leaf is not None:
                spanners = inspect_(previous_leaf).get_spanners(type(self))
                spanners = tuple(spanners)
                if spanners:
                    if spanners[0].clef == self.clef:
                        set_clef = False
        elif self._is_my_last_leaf(leaf):
            if self.clef != current_clef and current_clef is not None:
                reset_clef = True

            next_leaf = inspect_(leaf).get_leaf(1)
            while not isinstance(next_leaf, prototype):
                next_leaf = inspect_(next_leaf).get_leaf(1)
            if next_leaf is not None:
                spanners = inspect_(next_leaf).get_spanners(type(self))
                spanners = tuple(spanners)
                if spanners:
                    if spanners[0].clef == self.clef:
                        reset_clef = False
        if set_clef:
            string = format(self.clef, 'lilypond')
            lilypond_format_bundle.before.indicators.append(string)
        if reset_clef and current_clef is not None:
            string = format(current_clef, 'lilypond')
            lilypond_format_bundle.after.indicators.append(string)
        return lilypond_format_bundle

    ### PUBLIC PROPERTIES ###

    @property
    def clef(self):
        r'''Gets clef.

        Set to clef or clef name.

        Defaults to ``'percussion'``.

        Returns clef.
        '''
        return self._clef



# -*- coding: utf-8 -*-
from abjad.demos import part


def test_demos_part_durate_pitch_contour_reservoir_01():

    pitch_contour_reservoir = part.create_pitch_contour_reservoir()
    shadowed_reservoir = part.shadow_pitch_contour_reservoir(
        pitch_contour_reservoir)
    durated_reservoir = part.durate_pitch_contour_reservoir(
        shadowed_reservoir)



# -*- coding: utf-8 -*-
from abjad.tools.abctools.AbjadValueObject import AbjadValueObject


class BreathMark(AbjadValueObject):
    r'''Breath mark.

    ..  container:: example

        **Example 1.** Attached to a single note:

        ::

            >>> note = Note("c'4")
            >>> breath_mark = indicatortools.BreathMark()
            >>> attach(breath_mark, note)
            >>> show(note) # doctest: +SKIP

        ..  doctest::

            >>> print(format(note))
            c'4 \breathe

    ..  container:: example

        **Examle 2.** Attached to notes in a staff:

        ::

            >>> staff = Staff("c'8 d' e' f' g' a' b' c''")
            >>> attach(Beam(), staff[:4])
            >>> attach(Beam(), staff[4:])
            >>> attach(indicatortools.BreathMark(), staff[3])
            >>> attach(indicatortools.BreathMark(), staff[7])
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> print(format(staff))
            \new Staff {
                c'8 [
                d'8
                e'8
                f'8 ]
                \breathe
                g'8 [
                a'8
                b'8
                c''8 ]
                \breathe
            }

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_default_scope',
        )

    _format_slot = 'after'

    ### INITIALIZER ###

    def __init__(self):
        self._default_scope = None

    ### SPECIAL METHODS ###

    def __str__(self):
        r'''Gets string representation of breath mark.

        ..  container:: example

            ::

                >>> str(indicatortools.BreathMark())
                '\\breathe'

        Returns string.
        '''
        return r'\breathe'

    ### PRIVATE PROPERTIES ###

    @property
    def _contents_repr_string(self):
        return str(self)

    ### PRIVATE METHODS ###

    def _get_lilypond_format(self):
        return str(self)

    def _get_lilypond_format_bundle(self, component=None):
        from abjad.tools import systemtools
        lilypond_format_bundle = systemtools.LilyPondFormatBundle()
        lilypond_format_bundle.after.commands.append(str(self))
        return lilypond_format_bundle

    ### PUBLIC PROPERTIES ###

    @property
    def default_scope(self):
        r'''Gets default scope of breath mark.

        ..  container:: example

            ::

                >>> breath_mark = indicatortools.BreathMark()
                >>> breath_mark.default_scope is None
                True

        Returns none.
        '''
        return self._default_scope



#! /usr/bin/env python
# -*- coding: utf-8 -*-
import sys
import time


interval = int(sys.argv[1])
i = 0
while True:
    time.sleep(interval)
    i += interval
    unit = 'seconds'
    if i == 1:
        unit = 'second'
    message = '\n[{} {}]'
    message = message.format(i, unit)
    print(message)



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_Tuplet___init___01():
    r'''Initializes tuplet from empty input.
    '''

    tuplet = Tuplet()

    assert format(tuplet) == '\\times 2/3 {\n}'
    assert tuplet.multiplier == Multiplier(2, 3)
    assert not len(tuplet)



# -*- coding: utf-8 -*-


def select(indices=None, inverted=None):
    r'''Makes pattern that matches `indices`.

    ..  container:: example

        Selects index 2:

        ::

            >>> pattern = patterntools.select([2])

        ::

            >>> print(format(pattern))
            patterntools.Pattern(
                indices=[2],
                )

    ..  container:: example

        Selects indices 2, 3 and 5:

        ::

            >>> pattern = patterntools.select([2, 3, 5])

        ::

            >>> print(format(pattern))
            patterntools.Pattern(
                indices=[2, 3, 5],
                )

    Returns pattern.
    '''
    from abjad.tools import patterntools
    indices = indices or []
    return patterntools.Pattern(
        indices=indices,
        inverted=inverted,
        )



# -*- coding: utf-8 -*-
from abjad import *


def test_datastructuretools_OrdinalConstant___format___01():
    r'''Ordinal constants are storable and the format is evaluable.
    '''

    vector_constant_1 = datastructuretools.OrdinalConstant('x', -1, 'Left')
    string = format(vector_constant_1)

    assert string == 'Left'
    vector_constant_2 = eval(string)

    assert isinstance(vector_constant_1, datastructuretools.OrdinalConstant)
    assert isinstance(vector_constant_2, datastructuretools.OrdinalConstant)
    assert not vector_constant_1 is vector_constant_2
    assert vector_constant_1 == vector_constant_2



# -*- coding: utf-8 -*-
import abjad
from abjad.tools import indicatortools
from abjad.tools import instrumenttools
from abjad.tools import scoretools
from abjad.tools import templatetools
from abjad.tools.topleveltools import attach
from abjad.tools.topleveltools import detach


def make_mozart_score():
    r'''Makes Mozart score.
    '''

    score_template = templatetools.TwoStaffPianoScoreTemplate()
    score = score_template()

    # select the measures to use
    choices = abjad.demos.mozart.choose_mozart_measures()

    # create and populate the volta containers
    treble_volta = scoretools.Container()
    bass_volta = scoretools.Container()
    for choice in choices[:7]:
        treble, bass = abjad.demos.mozart.make_mozart_measure(choice)
        treble_volta.append(treble)
        bass_volta.append(bass)

    # attach indicators to the volta containers
    command = indicatortools.LilyPondCommand(
        'repeat volta 2', 'before'
        )
    attach(command, treble_volta)
    command = indicatortools.LilyPondCommand(
        'repeat volta 2', 'before'
        )
    attach(command, bass_volta)

    # append the volta containers to our staves
    score['RH Voice'].append(treble_volta)
    score['LH Voice'].append(bass_volta)

    # create and populate the alternative ending containers
    treble_alternative = scoretools.Container()
    bass_alternative = scoretools.Container()
    for choice in choices[7:9]:
        treble, bass = abjad.demos.mozart.make_mozart_measure(choice)
        treble_alternative.append(treble)
        bass_alternative.append(bass)

    # attach indicators to the alternative containers
    command = indicatortools.LilyPondCommand(
        'alternative', 'before'
        )
    attach(command, treble_alternative)
    command = indicatortools.LilyPondCommand(
        'alternative', 'before'
        )
    attach(command, bass_alternative)

    # append the alternative containers to our staves
    score['RH Voice'].append(treble_alternative)
    score['LH Voice'].append(bass_alternative)

    # create the remaining measures
    for choice in choices[9:]:
        treble, bass = abjad.demos.mozart.make_mozart_measure(choice)
        score['RH Voice'].append(treble)
        score['LH Voice'].append(bass)

    # attach indicators
    time_signature = indicatortools.TimeSignature((3, 8))
    attach(time_signature, score['RH Staff'])
    bar_line = indicatortools.BarLine('|.')
    attach(bar_line, score['RH Voice'][-1])
    bar_line = indicatortools.BarLine('|.')
    attach(bar_line, score['LH Voice'][-1])

    # remove the old, default piano instrument attached to the piano staff
    # and attach a custom instrument mark
    detach(instrumenttools.Instrument, score['Piano Staff'])

    klavier = instrumenttools.Piano(
        instrument_name='Katzenklavier',
        short_instrument_name='kk.',
        )
    attach(klavier, score['Piano Staff'])

    return score


# -*- coding: utf-8 -*-


def add_terminal_newlines(lines):
    r'''Adds terminal newlines to `lines`.

    ..  container:: example

        ::

            >>> lines = ['first line', 'second line']
            >>> stringtools.add_terminal_newlines(lines)
            ['first line\n', 'second line\n']

    Does nothing when line in `lines` already ends in newline:

    ..  container:: example

        ::

            >>> lines = ['first line\n', 'second line\n']
            >>> stringtools.add_terminal_newlines(lines)
            ['first line\n', 'second line\n']

    Returns newly constructed object of `lines` type.
    '''
    terminated_lines = []
    for line in lines:
        if not line.endswith('\n'):
            line = line + '\n'
        terminated_lines.append(line)
    terminated_lines = type(lines)(terminated_lines)
    return terminated_lines



# -*- coding: utf-8 -*-
from abjad import *


def test_durationtools_Duration___sub___01():
    r'''Subtracting nonreduced fraction from duration
    returns nonreduced fraction.
    '''

    result = Duration(1, 2) - mathtools.NonreducedFraction(2, 8)
    assert isinstance(result, mathtools.NonreducedFraction)
    assert result.pair == (2, 8)


def test_durationtools_Duration___sub___02():
    r'''Subtracting duration from nonreduced fraction
    returns nonreduced fraction.
    '''

    result = mathtools.NonreducedFraction(4, 8) - Duration(1, 4)
    assert isinstance(result, mathtools.NonreducedFraction)
    assert result.pair == (2, 8)



# -*- coding: utf-8 -*-
from abjad.tools import rhythmtreetools


def test_rhythmtreetools_RhythmTreeNode_parentage_ratios_01():

    string = '(1 (1 (2 (3 4)) 2))'
    tree = rhythmtreetools.RhythmTreeParser()(string)[0]

    assert tree.parentage_ratios == (1,)
    assert tree[0].parentage_ratios == (1, (1, 5))
    assert tree[1].parentage_ratios == (1, (2, 5))
    assert tree[1][0].parentage_ratios == (1, (2, 5), (3, 7))
    assert tree[1][1].parentage_ratios == (1, (2, 5), (4, 7))
    assert tree[2].parentage_ratios == (1, (2, 5))



# -*- coding: utf-8 -*-
from abjad.tools.spannertools.Spanner import Spanner


class HiddenStaffSpanner(Spanner):
    r'''Hidden staff spanner.

    ..  container:: example

        ::

            >>> staff = Staff("c'8 d'8 e'8 f'8")
            >>> spanner = spannertools.HiddenStaffSpanner()
            >>> attach(spanner, staff[1:3])
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                c'8
                \stopStaff
                d'8
                e'8
                \startStaff
                f'8
            }

    Formats LilyPond ``\stopStaff`` before first leaf in spanner.

    Formats LilyPond ``\startStaff`` command after last leaf in spanner.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        )

    ### INITIALIZER ###

    def __init__(
        self,
        overrides=None,
        ):
        Spanner.__init__(
            self,
            overrides=overrides,
            )

    ### PRIVATE METHODS ###

    def _format_after_leaf(self, leaf):
        result = []
        if self._is_my_last_leaf(leaf):
            result.append(r'\startStaff')
        return result

    def _format_before_leaf(self, leaf):
        result = []
        if self._is_my_first_leaf(leaf):
            result.append(r'\stopStaff')
        return result
